@@ -18,7 +18,6 @@ from oslo_utils import uuidutils
 from senlin.common import consts
 from senlin.common import context as senlin_context
 from senlin.common import exception
-from senlin.common.i18n import _
 from senlin.common import utils
 from senlin.drivers import base as driver_base
 from senlin.objects import credential as co
@@ -68,11 +67,12 @@ class Receiver(object):
         self.params = kwargs.get('params', {})
         self.channel = kwargs.get('channel', {})
 
-    def store(self, context):
+    def store(self, context, update=False):
         """Store the receiver in database and return its ID.
 
         :param context: Context for DB operations.
         """
+        timestamp = timeutils.utcnow(True)
         self.created_at = timeutils.utcnow(True)
         values = {
             'id': self.id,
@@ -90,9 +90,15 @@ class Receiver(object):
             'channel': self.channel,
         }
 
-        # TODO(Qiming): Add support to update
-        receiver = ro.Receiver.create(context, values)
-        self.id = receiver.id
+        if update:
+            self.updated_at = timestamp
+            values['updated_at'] = timestamp
+            ro.Receiver.update(context, self.id, values)
+        else:
+            self.created_at = timestamp
+            values['created_at'] = timestamp
+            receiver = ro.Receiver.create(context, values)
+            self.id = receiver.id
 
         return self.id
 
@@ -108,11 +114,11 @@ class Receiver(object):
             # otherwise, use context user
             cdata['trust_id'] = context.trusts
 
-        kwargs['id'] = uuidutils.generate_uuid()
         kwargs['actor'] = cdata
-        kwargs['user'] = context.user
-        kwargs['project'] = context.project
-        kwargs['domain'] = context.domain
+        kwargs['user'] = context.user_id
+        kwargs['project'] = context.project_id
+        kwargs['domain'] = context.domain_id
+        kwargs['id'] = uuidutils.generate_uuid()
         cluster_id = cluster.id if cluster else None
         obj = cls(rtype, cluster_id, action, **kwargs)
         obj.initialize_channel(context)
@@ -164,18 +170,6 @@ class Receiver(object):
 
         return cls._from_object(receiver_obj)
 
-    @classmethod
-    def load_all(cls, context, limit=None, marker=None, sort=None,
-                 filters=None, project_safe=True):
-        """Retrieve all receivers from database."""
-
-        objs = ro.Receiver.get_all(context, limit=limit, marker=marker,
-                                   sort=sort, filters=filters,
-                                   project_safe=project_safe)
-
-        for obj in objs:
-            yield cls._from_object(obj)
-
     def to_dict(self):
         info = {
             'id': self.id,
@@ -218,13 +212,12 @@ class Receiver(object):
 
     def _get_base_url(self):
         base = None
-        service_cred = senlin_context.get_service_context()
+        service_cred = senlin_context.get_service_credentials()
         kc = driver_base.SenlinDriver().identity(service_cred)
         try:
             base = kc.get_senlin_endpoint()
         except exception.InternalError as ex:
-            msg = _('Senlin endpoint can not be found: %s.') % ex.message
-            LOG.warning(msg)
+            LOG.warning('Senlin endpoint can not be found: %s.', ex)
 
         return base
 
@@ -236,7 +229,7 @@ class Receiver(object):
         :returns: A dict containing the required parameters for connection
                   creation.
         """
-        service_creds = senlin_context.get_service_context()
+        service_creds = senlin_context.get_service_credentials()
         params = {
             'username': service_creds.get('username'),
             'password': service_creds.get('password'),
@@ -244,8 +237,7 @@ class Receiver(object):
             'user_domain_name': service_creds.get('user_domain_name')
         }
 
-        cred = co.Credential.get(oslo_context.get_current(),
-                                 user, project)
+        cred = co.Credential.get(oslo_context.get_current(), user, project)
         if cred is None:
             raise exception.TrustNotFound(trustor=user)
         params['trust_id'] = cred.cred['openstack']['trust']
