@@ -42,7 +42,7 @@ class LoadBalancingPolicy(base.Policy):
     the cluster (which could be created by the policy) when these actions are
     performed.
     """
-    VERSION = '1.1'
+    VERSION = '1.3'
     VERSIONS = {
         '1.0': [
             {'status': consts.SUPPORTED, 'since': '2016.04'}
@@ -50,6 +50,12 @@ class LoadBalancingPolicy(base.Policy):
         '1.1': [
             {'status': consts.SUPPORTED, 'since': '2018.01'}
         ],
+        '1.2': [
+            {'status': consts.SUPPORTED, 'since': '2020.02'}
+        ],
+        '1.3': [
+            {'status': consts.SUPPORTED, 'since': '2020.03'}
+        ],
     }
 
     PRIORITY = 500
@@ -60,10 +66,12 @@ class LoadBalancingPolicy(base.Policy):
         ('AFTER', consts.CLUSTER_RESIZE),
         ('AFTER', consts.NODE_RECOVER),
         ('AFTER', consts.NODE_CREATE),
+        ('AFTER', consts.CLUSTER_REPLACE_NODES),
         ('BEFORE', consts.CLUSTER_DEL_NODES),
         ('BEFORE', consts.CLUSTER_SCALE_IN),
         ('BEFORE', consts.CLUSTER_RESIZE),
         ('BEFORE', consts.NODE_DELETE),
+        ('BEFORE', consts.CLUSTER_REPLACE_NODES),
     ]
 
     PROFILE_TYPE = [
@@ -71,9 +79,11 @@ class LoadBalancingPolicy(base.Policy):
     ]
 
     KEYS = (
-        POOL, VIP, HEALTH_MONITOR, LB_STATUS_TIMEOUT, LOADBALANCER
+        POOL, VIP, HEALTH_MONITOR, LB_STATUS_TIMEOUT, LOADBALANCER,
+        AVAILABILITY_ZONE
     ) = (
-        'pool', 'vip', 'health_monitor', 'lb_status_timeout', 'loadbalancer'
+        'pool', 'vip', 'health_monitor', 'lb_status_timeout', 'loadbalancer',
+        'availability_zone'
     )
 
     _POOL_KEYS = (
@@ -109,10 +119,10 @@ class LoadBalancingPolicy(base.Policy):
     )
 
     _VIP_KEYS = (
-        VIP_SUBNET, VIP_ADDRESS, VIP_CONNECTION_LIMIT, VIP_PROTOCOL,
-        VIP_PROTOCOL_PORT, VIP_ADMIN_STATE_UP,
+        VIP_SUBNET, VIP_NETWORK, VIP_ADDRESS, VIP_CONNECTION_LIMIT,
+        VIP_PROTOCOL, VIP_PROTOCOL_PORT, VIP_ADMIN_STATE_UP,
     ) = (
-        'subnet', 'address', 'connection_limit', 'protocol',
+        'subnet', 'network', 'address', 'connection_limit', 'protocol',
         'protocol_port', 'admin_state_up',
     )
 
@@ -194,8 +204,13 @@ class LoadBalancingPolicy(base.Policy):
             schema={
                 VIP_SUBNET: schema.String(
                     _('Name or ID of Subnet on which the VIP address will be '
-                      'allocated.'),
-                    required=True,
+                      'allocated. One of Subnet or Network is required.'),
+                    required=False,
+                ),
+                VIP_NETWORK: schema.String(
+                    _('Name or ID of Network on which the VIP address will be '
+                      'allocated. One of Subnet or Network is required.'),
+                    required=False,
                 ),
                 VIP_ADDRESS: schema.String(
                     _('IP address of the VIP.'),
@@ -281,6 +296,11 @@ class LoadBalancingPolicy(base.Policy):
             _('Name or ID of loadbalancer for the cluster on which nodes can '
               'be connected.'),
             default=None,
+        ),
+        AVAILABILITY_ZONE: schema.String(
+            _('Name of the loadbalancer availability zone to use for creation '
+              'of the loadbalancer.'),
+            default=None,
         )
     }
 
@@ -290,6 +310,7 @@ class LoadBalancingPolicy(base.Policy):
         self.pool_spec = self.properties.get(self.POOL, {})
         self.vip_spec = self.properties.get(self.VIP, {})
         self.hm_spec = self.properties.get(self.HEALTH_MONITOR, None)
+        self.az_spec = self.properties.get(self.AVAILABILITY_ZONE, None)
         self.lb_status_timeout = self.properties.get(self.LB_STATUS_TIMEOUT)
         self.lb = self.properties.get(self.LOADBALANCER, None)
 
@@ -311,13 +332,33 @@ class LoadBalancingPolicy(base.Policy):
                     ) % {'key': self.POOL_SUBNET, 'value': name_or_id}
             raise exc.InvalidSpec(message=msg)
 
-        # validate VIP subnet
-        name_or_id = self.vip_spec.get(self.VIP_SUBNET)
+        # validate VIP subnet or network
+        subnet_name_or_id = self.vip_spec.get(self.VIP_SUBNET)
+        network_name_or_id = self.vip_spec.get(self.VIP_NETWORK)
+        if not subnet_name_or_id and not network_name_or_id:
+            msg = _("At least one of VIP Subnet or Network must be defined.")
+            raise exc.InvalidSpec(message=msg)
         try:
-            nc.subnet_get(name_or_id)
+            # Check subnet if it is set
+            obj_type = self.VIP_SUBNET
+            name_or_id = subnet_name_or_id
+            if name_or_id:
+                nc.subnet_get(name_or_id)
+
+            # Check network if it is set
+            obj_type = self.VIP_NETWORK
+            name_or_id = network_name_or_id
+            if name_or_id:
+                nc.network_get(name_or_id)
+
+            # TODO(rm_work): We *could* do more validation here to catch issues
+            # at validation time, like verifying the subnet's network_id is the
+            # same as the id of the network, if both are set -- but for now we
+            # will just leave that up to the LB API, which means if there is a
+            # failure, it won't be caught until attach time.
         except exc.InternalError:
             msg = _("The specified %(key)s '%(value)s' could not be found."
-                    ) % {'key': self.VIP_SUBNET, 'value': name_or_id}
+                    ) % {'key': obj_type, 'value': name_or_id}
             raise exc.InvalidSpec(message=msg)
 
         # validate loadbalancer
@@ -357,7 +398,7 @@ class LoadBalancingPolicy(base.Policy):
                 data['healthmonitor'] = self.hm_spec.get(self.HM_ID)
         else:
             res, data = lb_driver.lb_create(self.vip_spec, self.pool_spec,
-                                            self.hm_spec)
+                                            self.hm_spec, self.az_spec)
             if res is False:
                 return False, data
 
@@ -463,6 +504,9 @@ class LoadBalancingPolicy(base.Policy):
                     count = action.data['deletion']['count']
             else:  # action.action == consts.CLUSTER_SCALE_IN
                 count = 1
+        elif action.action == consts.CLUSTER_REPLACE_NODES:
+            candidates = list(action.inputs['candidates'].keys())
+            count = len(candidates)
         else:
             count = deletion.get('count', 0)
             candidates = deletion.get('candidates', None)
@@ -550,10 +594,11 @@ class LoadBalancingPolicy(base.Policy):
 
     def _get_post_candidates(self, action):
         # This method will parse action data passed from action layer
-
         if (action.action == consts.NODE_CREATE or
                 action.action == consts.NODE_RECOVER):
             candidates = [action.entity.id]
+        elif action.action == consts.CLUSTER_REPLACE_NODES:
+            candidates = list(action.inputs['candidates'].values())
         else:
             creation = action.data.get('creation', None)
             candidates = creation.get('nodes', []) if creation else []
@@ -571,7 +616,7 @@ class LoadBalancingPolicy(base.Policy):
         # lb_member is None, need to add to lb pool
         if not lb_member:
             values['data'] = data
-            no.Node.update(action.context, values)
+            no.Node.update(action.context, node.id, values)
             return candidates
 
         # was a member of lb pool, check whether has been recreated
@@ -580,7 +625,7 @@ class LoadBalancingPolicy(base.Policy):
                                 handle_err=False)
             data.pop('lb_member', None)
             values['data'] = data
-            no.Node.update(action.context, values)
+            no.Node.update(action.context, node.id, values)
             return candidates
 
         return None
@@ -637,10 +682,15 @@ class LoadBalancingPolicy(base.Policy):
         :returns: Nothing.
         """
 
+        # skip post op if action did not complete successfully
+        action_result = action.inputs.get('action_result', 'OK')
+        if action_result != 'OK':
+            return
+
         # TODO(Yanyanhu): Need special handling for cross-az scenario
         # which is supported by Neutron lbaas.
         candidates = self._get_post_candidates(action)
-        if len(candidates) == 0:
+        if not candidates:
             return
 
         obj = action.entity
@@ -661,5 +711,3 @@ class LoadBalancingPolicy(base.Policy):
             error = _('Failed in adding nodes into lb pool: %s') % failed_nodes
             action.data['status'] = base.CHECK_ERROR
             action.data['reason'] = error
-
-        return
