@@ -13,9 +13,9 @@
 import base64
 import copy
 
+from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
-import six
 
 from senlin.common import constraints
 from senlin.common import consts
@@ -248,8 +248,6 @@ class ServerProfile(base.Profile):
         'rescue', 'unrescue', 'evacuate', 'migrate',
     )
 
-    REBOOT_TYPE = 'type'
-    REBOOT_TYPES = (REBOOT_SOFT, REBOOT_HARD) = ('SOFT', 'HARD')
     ADMIN_PASSWORD = 'admin_pass'
     RESCUE_IMAGE = 'image_ref'
     EVACUATE_OPTIONS = (
@@ -262,11 +260,11 @@ class ServerProfile(base.Profile):
         OP_REBOOT: schema.Operation(
             _("Reboot the nova server."),
             schema={
-                REBOOT_TYPE: schema.StringParam(
+                consts.REBOOT_TYPE: schema.StringParam(
                     _("Type of reboot which can be 'SOFT' or 'HARD'."),
-                    default=REBOOT_SOFT,
+                    default=consts.REBOOT_SOFT,
                     constraints=[
-                        constraints.AllowedValues(REBOOT_TYPES),
+                        constraints.AllowedValues(consts.REBOOT_TYPES),
                     ]
                 )
             }
@@ -340,7 +338,7 @@ class ServerProfile(base.Profile):
         except exc.InternalError as ex:
             if reason == 'create':
                 raise exc.EResourceCreation(type='server',
-                                            message=six.text_type(ex))
+                                            message=str(ex))
             else:
                 raise
 
@@ -360,7 +358,7 @@ class ServerProfile(base.Profile):
         try:
             flavor = self.compute(obj).flavor_find(name_or_id, False)
         except exc.InternalError as ex:
-            msg = six.text_type(ex)
+            msg = str(ex)
             if reason is None:  # reason is 'validate'
                 if ex.code == 404:
                     msg = _("The specified %(k)s '%(v)s' could not be found."
@@ -385,14 +383,14 @@ class ServerProfile(base.Profile):
 
     def _validate_image(self, obj, name_or_id, reason=None):
         try:
-            return self.compute(obj).image_find(name_or_id, False)
+            return self.glance(obj).image_find(name_or_id, False)
         except exc.InternalError as ex:
             if reason == 'create':
                 raise exc.EResourceCreation(type='server',
-                                            message=six.text_type(ex))
+                                            message=str(ex))
             elif reason == 'update':
                 raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                          message=six.text_type(ex))
+                                          message=str(ex))
             elif ex.code == 404:
                 msg = _("The specified %(k)s '%(v)s' could not be found."
                         ) % {'k': self.IMAGE, 'v': name_or_id}
@@ -406,10 +404,10 @@ class ServerProfile(base.Profile):
         except exc.InternalError as ex:
             if reason == 'create':
                 raise exc.EResourceCreation(type='server',
-                                            message=six.text_type(ex))
+                                            message=str(ex))
             elif reason == 'update':
                 raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                          message=six.text_type(ex))
+                                          message=str(ex))
             elif ex.code == 404:
                 msg = _("The specified %(k)s '%(v)s' could not be found."
                         ) % {'k': self.KEY_NAME, 'v': name_or_id}
@@ -430,7 +428,7 @@ class ServerProfile(base.Profile):
         except exc.InternalError as ex:
             if reason == 'create':
                 raise exc.EResourceCreation(type='server',
-                                            message=six.text_type(ex))
+                                            message=str(ex))
             elif ex.code == 404:
                 msg = _("The specified volume '%(k)s' could not be found."
                         ) % {'k': name_or_id}
@@ -497,10 +495,16 @@ class ServerProfile(base.Profile):
         res = []
         try:
             for sg in sgs:
-                sg_obj = nc.security_group_find(sg)
+                try:
+                    # try to find sg scoped by project first
+                    sg_obj = nc.security_group_find(
+                        sg, project_id=self.project)
+                except exc.InternalError:
+                    # if it fails to find sg, try without project scope
+                    sg_obj = nc.security_group_find(sg)
                 res.append(sg_obj.id)
         except exc.InternalError as ex:
-            return six.text_type(ex)
+            return str(ex)
 
         result[self.PORT_SECURITY_GROUPS] = res
         return
@@ -517,9 +521,11 @@ class ServerProfile(base.Profile):
             return
         try:
             net_obj = nc.network_get(net)
+            if net_obj is None:
+                return _("The specified network %s could not be found.") % net
             result[self.NETWORK] = net_obj.id
         except exc.InternalError as ex:
-            return six.text_type(ex)
+            return str(ex)
 
     def _check_port(self, nc, port, result):
         """Check the specified port.
@@ -540,7 +546,7 @@ class ServerProfile(base.Profile):
             result[self.PORT] = port_obj.id
             return
         except exc.InternalError as ex:
-            return six.text_type(ex)
+            return str(ex)
 
     def _check_floating_ip(self, nc, net_spec, result):
         """Check floating IP and network, if specified.
@@ -554,9 +560,12 @@ class ServerProfile(base.Profile):
         if net:
             try:
                 net_obj = nc.network_get(net)
+                if net_obj is None:
+                    return _("The floating network %s could not be found."
+                             ) % net
                 result[self.FLOATING_NETWORK] = net_obj.id
             except exc.InternalError as ex:
-                return six.text_type(ex)
+                return str(ex)
 
         flt_ip = net_spec.get(self.FLOATING_IP)
         if not flt_ip:
@@ -577,7 +586,7 @@ class ServerProfile(base.Profile):
             result[self.FLOATING_IP] = flt_ip
             return
         except exc.InternalError as ex:
-            return six.text_type(ex)
+            return str(ex)
 
     def _validate_network(self, obj, net_spec, reason=None):
 
@@ -764,9 +773,13 @@ class ServerProfile(base.Profile):
                     port_attrs['floating'].update({'remove': True})
             internal_ports.append(port_attrs)
         if internal_ports:
-            node_data = obj.data
-            node_data.update(internal_ports=internal_ports)
-            node_obj.Node.update(self.context, obj.id, {'data': node_data})
+            try:
+                node_data = obj.data
+                node_data.update(internal_ports=internal_ports)
+                node_obj.Node.update(self.context, obj.id, {'data': node_data})
+            except exc.ResourceNotFound:
+                self._rollback_ports(obj, internal_ports)
+                raise
         return internal_ports
 
     def _build_metadata(self, obj, usermeta):
@@ -779,7 +792,7 @@ class ServerProfile(base.Profile):
         metadata['cluster_node_id'] = obj.id
         if obj.cluster_id:
             metadata['cluster_id'] = obj.cluster_id
-            metadata['cluster_node_index'] = six.text_type(obj.index)
+            metadata['cluster_node_index'] = str(obj.index)
 
         return metadata
 
@@ -805,14 +818,7 @@ class ServerProfile(base.Profile):
 
         :param obj: The node object for which a server will be created.
         """
-        kwargs = {}
-        for key in self.KEYS:
-            # context is treated as connection parameters
-            if key == self.CONTEXT:
-                continue
-
-            if self.properties[key] is not None:
-                kwargs[key] = self.properties[key]
+        kwargs = self._generate_kwargs()
 
         admin_pass = self.properties[self.ADMIN_PASS]
         if admin_pass:
@@ -855,6 +861,7 @@ class ServerProfile(base.Profile):
             kwargs['user_data'] = encodeutils.safe_decode(base64.b64encode(ud))
 
         networks = self.properties[self.NETWORKS]
+        ports = None
         if networks is not None:
             ports = self._create_ports_from_properties(
                 obj, networks, 'create')
@@ -879,18 +886,103 @@ class ServerProfile(base.Profile):
         resource_id = None
         try:
             server = self.compute(obj).server_create(**kwargs)
-            self.compute(obj).wait_for_server(server.id)
+            self.compute(obj).wait_for_server(
+                server.id, timeout=cfg.CONF.default_nova_timeout)
             server = self.compute(obj).server_get(server.id)
             # Update zone placement info if available
             self._update_zone_info(obj, server)
             return server.id
+        except exc.ResourceNotFound:
+            self._rollback_ports(obj, ports)
+            self._rollback_instance(obj, server)
+            raise
         except exc.InternalError as ex:
             if server and server.id:
                 resource_id = server.id
+                LOG.debug('Deleting server %s that is ERROR state after'
+                          ' create.', server.id)
+
+                try:
+                    obj.physical_id = server.id
+                    self.do_delete(obj, internal_ports=ports)
+                except Exception:
+                    LOG.error('Failed to delete server %s', server.id)
+                    pass
+            elif ports:
+                self._delete_ports(obj, ports)
             raise exc.EResourceCreation(type='server',
-                                        message=six.text_type(ex),
+                                        message=str(ex),
                                         resource_id=resource_id)
 
+    def _generate_kwargs(self):
+        """Generate the base kwargs for a server.
+
+        :return:
+        """
+        kwargs = {}
+        for key in self.KEYS:
+            # context is treated as connection parameters
+            if key == self.CONTEXT:
+                continue
+
+            if self.properties[key] is not None:
+                kwargs[key] = self.properties[key]
+        return kwargs
+
+    def _rollback_ports(self, obj, ports):
+        """Rollback any ports created after a ResourceNotFound exception.
+
+        :param obj: The node object.
+        :param ports: A list of ports which attached to this server.
+        :return:
+        """
+        if not ports:
+            return
+
+        LOG.warning(
+            'Rolling back ports for Node %s.',
+            obj.id
+        )
+
+        for port in ports:
+            if not port.get('remove', False):
+                continue
+            try:
+                if (port.get('floating') and
+                        port['floating'].get('remove', False)):
+                    self.network(obj).floatingip_delete(
+                        port['floating']['id']
+                    )
+                self.network(obj).port_delete(port['id'])
+            except exc.InternalError as ex:
+                LOG.debug(
+                    'Failed to delete port %s during rollback for Node %s: %s',
+                    port['id'], obj.id, ex
+                )
+
+    def _rollback_instance(self, obj, server):
+        """Rollback an instance created after a ResourceNotFound exception.
+
+        :param obj: The node object.
+        :param server: A server.
+        :return:
+        """
+        if not server or not server.id:
+            return
+
+        LOG.warning(
+            'Rolling back instance %s for Node %s.',
+            server.id, obj.id
+        )
+
+        try:
+            self.compute(obj).server_force_delete(server.id, True)
+        except exc.InternalError as ex:
+            LOG.debug(
+                'Failed to delete instance %s during rollback for Node %s: %s',
+                server.id, obj.id, ex
+            )
+
     def do_delete(self, obj, **params):
         """Delete the physical resource associated with the specified node.
 
@@ -902,39 +994,29 @@ class ServerProfile(base.Profile):
         :raises: `EResourceDeletion` if interaction with compute service fails.
         """
         server_id = obj.physical_id
-        if not server_id:
-            return True
-
         ignore_missing = params.get('ignore_missing', True)
         internal_ports = obj.data.get('internal_ports', [])
         force = params.get('force', False)
-        timeout = params.get('timeout', None)
-        delete_ports_on_failure = params.get('delete_ports_on_failure', False)
+        timeout = params.get('timeout', cfg.CONF.default_nova_timeout)
 
         try:
-            driver = self.compute(obj)
-            if force:
-                driver.server_force_delete(server_id, ignore_missing)
-            else:
-                driver.server_delete(server_id, ignore_missing)
+            if server_id:
+                driver = self.compute(obj)
+                if force:
+                    driver.server_force_delete(server_id, ignore_missing)
+                else:
+                    driver.server_delete(server_id, ignore_missing)
+
+                driver.wait_for_server_delete(server_id, timeout=timeout)
         except exc.InternalError as ex:
             raise exc.EResourceDeletion(type='server', id=server_id,
-                                        message=six.text_type(ex))
-
-        try:
-            driver.wait_for_server_delete(server_id, timeout)
-        except exc.InternalError as ex:
-            if delete_ports_on_failure and internal_ports:
-                del_ports_ex = self._delete_ports(obj, internal_ports)
-                if del_ports_ex:
-                    raise del_ports_ex
-            raise exc.EResourceDeletion(type='server', id=server_id,
-                                        message=six.text_type(ex))
-
-        if internal_ports:
-            ex = self._delete_ports(obj, internal_ports)
-            if ex:
-                raise ex
+                                        message=str(ex))
+        finally:
+            if internal_ports:
+                ex = self._delete_ports(obj, internal_ports)
+                if ex:
+                    raise exc.EResourceDeletion(type='server', d=server_id,
+                                                message=str(ex))
         return True
 
     def _check_server_name(self, obj, profile):
@@ -964,7 +1046,7 @@ class ServerProfile(base.Profile):
             self.compute(obj).server_update(obj.physical_id, name=new_name)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                      message=six.text_type(ex))
+                                      message=str(ex))
 
     def _check_password(self, obj, new_profile):
         """Check if the admin password has been changed in the new profile.
@@ -995,7 +1077,7 @@ class ServerProfile(base.Profile):
                                                      new_password)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                      message=six.text_type(ex))
+                                      message=str(ex))
 
     def _update_metadata(self, obj, new_profile):
         """Update the server metadata.
@@ -1016,7 +1098,7 @@ class ServerProfile(base.Profile):
             self.compute(obj).server_metadata_update(obj.physical_id, new_meta)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                      message=six.text_type(ex))
+                                      message=str(ex))
 
     def _update_flavor(self, obj, new_profile):
         """Update server flavor.
@@ -1039,12 +1121,12 @@ class ServerProfile(base.Profile):
             cc.server_resize(obj.physical_id, newflavor.id)
             cc.wait_for_server(obj.physical_id, 'VERIFY_RESIZE')
         except exc.InternalError as ex:
-            msg = six.text_type(ex)
+            msg = str(ex)
             try:
                 cc.server_resize_revert(obj.physical_id)
                 cc.wait_for_server(obj.physical_id, consts.VS_ACTIVE)
             except exc.InternalError as ex1:
-                msg = six.text_type(ex1)
+                msg = str(ex1)
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
                                       message=msg)
 
@@ -1053,7 +1135,7 @@ class ServerProfile(base.Profile):
             cc.wait_for_server(obj.physical_id, consts.VS_ACTIVE)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                      message=six.text_type(ex))
+                                      message=str(ex))
 
     def _update_image(self, obj, new_profile, new_name, new_password):
         """Update image used by server node.
@@ -1083,7 +1165,7 @@ class ServerProfile(base.Profile):
             server = driver.server_get(obj.physical_id)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                      message=six.text_type(ex))
+                                      message=str(ex))
         old_image_id = self._get_image_id(obj, server, 'updating')
 
         if new_image_id == old_image_id:
@@ -1095,7 +1177,7 @@ class ServerProfile(base.Profile):
             driver.wait_for_server(obj.physical_id, consts.VS_ACTIVE)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                      message=six.text_type(ex))
+                                      message=str(ex))
         return True
 
     def _update_network_add_port(self, obj, networks):
@@ -1112,18 +1194,18 @@ class ServerProfile(base.Profile):
             server = cc.server_get(obj.physical_id)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                      message=six.text_type(ex))
+                                      message=str(ex))
 
         ports = self._create_ports_from_properties(
             obj, networks, 'update')
         for port in ports:
-            params = {'port': port['id']}
+            params = {'port_id': port['id']}
             try:
                 cc.server_interface_create(server, **params)
             except exc.InternalError as ex:
                 raise exc.EResourceUpdate(type='server',
                                           id=obj.physical_id,
-                                          message=six.text_type(ex))
+                                          message=str(ex))
 
     def _find_port_by_net_spec(self, obj, net_spec, ports):
         """Find existing ports match with specific network properties.
@@ -1197,7 +1279,7 @@ class ServerProfile(base.Profile):
                                          ignore_missing=True)
             except exc.InternalError as ex:
                 raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                          message=six.text_type(ex))
+                                          message=str(ex))
             internal_ports.remove(port)
         obj.data['internal_ports'] = internal_ports
         node_obj.Node.update(self.context, obj.id, {'data': obj.data})
@@ -1300,7 +1382,7 @@ class ServerProfile(base.Profile):
             return {
                 'Error': {
                     'code': ex.code,
-                    'message': six.text_type(ex)
+                    'message': str(ex)
                 }
             }
 
@@ -1319,7 +1401,7 @@ class ServerProfile(base.Profile):
         details = {
             'image': image_id,
             'attached_volumes': attached_volumes,
-            'flavor': server_data['flavor']['id'],
+            'flavor': self._get_flavor_id(obj, server_data),
         }
         for key in known_keys:
             if key in server_data:
@@ -1343,8 +1425,9 @@ class ServerProfile(base.Profile):
         # process security groups
         sgroups = []
         if 'security_groups' in server_data:
-            for sg in server_data['security_groups']:
-                sgroups.append(sg['name'])
+            if server_data['security_groups'] is not None:
+                for sg in server_data['security_groups']:
+                    sgroups.append(sg['name'])
         # when we have multiple nics the info will include the
         # security groups N times where N == number of nics. Be nice
         # and only display it once.
@@ -1358,6 +1441,20 @@ class ServerProfile(base.Profile):
 
         return dict((k, details[k]) for k in sorted(details))
 
+    def _get_flavor_id(self, obj, server):
+        """Get flavor id.
+
+        :param obj: The node object.
+        :param dict server: The server object.
+        :return: The flavor_id for the server.
+        """
+        flavor = server['flavor']
+
+        if 'id' in flavor:
+            return flavor['id']
+
+        return self.compute(obj).flavor_find(flavor['original_name'], False).id
+
     def _get_image_id(self, obj, server, op):
         """Get image id.
 
@@ -1384,7 +1481,7 @@ class ServerProfile(base.Profile):
                 except exc.InternalError as ex:
                     raise exc.EResourceOperation(op=op, type='server',
                                                  id=obj.physical_id,
-                                                 message=six.text_type(ex))
+                                                 message=str(ex))
         else:
             msg = _("server doesn't have an image and it has no "
                     "bootable volume")
@@ -1411,7 +1508,7 @@ class ServerProfile(base.Profile):
         except exc.InternalError as ex:
             raise exc.EResourceOperation(op=op_name, type='server',
                                          id=server_id,
-                                         message=six.text_type(ex))
+                                         message=str(ex))
 
     def do_adopt(self, obj, overrides=None, snapshot=False):
         """Adopt an existing server node for management.
@@ -1438,7 +1535,7 @@ class ServerProfile(base.Profile):
         try:
             server = driver.server_get(obj.physical_id)
         except exc.InternalError as ex:
-            error = {'code': ex.code, 'message': six.text_type(ex)}
+            error = {'code': ex.code, 'message': str(ex)}
 
         if error:
             return {'Error': error}
@@ -1499,13 +1596,13 @@ class ServerProfile(base.Profile):
 
         driver = self.compute(obj)
         try:
-            metadata = driver.server_metadata_get(obj.physical_id) or {}
+            metadata = {}
             metadata['cluster_id'] = cluster_id
-            metadata['cluster_node_index'] = six.text_type(obj.index)
+            metadata['cluster_node_index'] = str(obj.index)
             driver.server_metadata_update(obj.physical_id, metadata)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
-                                      message=six.text_type(ex))
+                                      message=str(ex))
         return super(ServerProfile, self).do_join(obj, cluster_id)
 
     def do_leave(self, obj):
@@ -1517,7 +1614,7 @@ class ServerProfile(base.Profile):
             self.compute(obj).server_metadata_delete(obj.physical_id, keys)
         except exc.InternalError as ex:
             raise exc.EResourceDeletion(type='server', id=obj.physical_id,
-                                        message=six.text_type(ex))
+                                        message=str(ex))
         return super(ServerProfile, self).do_leave(obj)
 
     def do_check(self, obj):
@@ -1527,20 +1624,77 @@ class ServerProfile(base.Profile):
         try:
             server = self.compute(obj).server_get(obj.physical_id)
         except exc.InternalError as ex:
-            if "No Server found" in six.text_type(ex):
+            if ex.code == 404:
                 raise exc.EServerNotFound(type='server',
                                           id=obj.physical_id,
-                                          message=six.text_type(ex))
+                                          message=str(ex))
             else:
                 raise exc.EResourceOperation(op='checking', type='server',
                                              id=obj.physical_id,
-                                             message=six.text_type(ex))
+                                             message=str(ex))
 
         if (server is None or server.status != consts.VS_ACTIVE):
             return False
 
         return True
 
+    def do_healthcheck(self, obj):
+        """Healthcheck operation.
+
+        This method checks if a server node is healthy by getting the server
+        status from nova.  A server is considered unhealthy if it does not
+        exist or its status is one of the following:
+        - ERROR
+        - SHUTOFF
+        - DELETED
+
+        :param obj: The node object to operate on.
+        :return status: True indicates node is healthy, False indicates
+            it is unhealthy.
+        """
+        unhealthy_server_status = [consts.VS_ERROR, consts.VS_SHUTOFF,
+                                   consts.VS_DELETED]
+
+        if not obj.physical_id:
+            if obj.status == 'BUILD' or obj.status == 'CREATING':
+                return True
+
+            LOG.info('%s for %s: server has no physical ID.',
+                     consts.POLL_STATUS_FAIL, obj.name)
+            return False
+
+        try:
+            server = self.compute(obj).server_get(obj.physical_id)
+        except Exception as ex:
+            if isinstance(ex, exc.InternalError) and ex.code == 404:
+                # treat resource not found exception as unhealthy
+                LOG.info('%s for %s: server was not found.',
+                         consts.POLL_STATUS_FAIL, obj.name)
+                return False
+            else:
+                # treat all other exceptions as healthy
+                LOG.info(
+                    '%s for %s: Exception when trying to get server info but '
+                    'ignoring this error: %s.',
+                    consts.POLL_STATUS_PASS, obj.name, ex.message)
+                return True
+
+        if server is None:
+            # no server information is available, treat the node as healthy
+            LOG.info(
+                '%s for %s: No server information was returned but ignoring '
+                'this error.',
+                consts.POLL_STATUS_PASS, obj.name)
+            return True
+
+        if server.status in unhealthy_server_status:
+            LOG.info('%s for %s: server status is unhealthy.',
+                     consts.POLL_STATUS_FAIL, obj.name)
+            return False
+
+        LOG.info('%s for %s', consts.POLL_STATUS_PASS, obj.name)
+        return True
+
     def do_recover(self, obj, **options):
         """Handler for recover operation.
 
@@ -1552,29 +1706,32 @@ class ServerProfile(base.Profile):
         :return status: True indicates successful recovery, False indicates
             failure.
         """
-        operation = options.get('operation', None)
 
-        if operation and not isinstance(operation, six.string_types):
-            operation = operation[0]
+        # default is recreate if not specified
+        if 'operation' not in options or not options['operation']:
+            options['operation'] = consts.RECOVER_RECREATE
 
-        if operation is not None and 'name' in operation:
-            op_name = operation['name']
-            if op_name.upper() != consts.RECOVER_RECREATE:
-                op_params = operation.get('params', {})
-                # nova recover operation always use hard reboot
-                # vm in error or stop status soft reboot can't succeed
-                if op_name.upper() == consts.RECOVER_REBOOT:
-                    if self.REBOOT_TYPE not in op_params:
-                        op_params[self.REBOOT_TYPE] = self.REBOOT_HARD
-                if op_name.lower() not in self.OP_NAMES:
-                    LOG.error("The operation '%s' is not supported",
-                              op_name)
-                    return obj.physical_id, False
+        operation = options.get('operation')
 
-                method = getattr(self, "handle_" + op_name.lower())
-                return method(obj, **op_params)
+        if operation.upper() not in consts.RECOVERY_ACTIONS:
+            LOG.error("The operation '%s' is not supported",
+                      operation)
+            return obj.physical_id, False
 
-        return super(ServerProfile, self).do_recover(obj, **options)
+        op_params = options.get('operation_params', {})
+        if operation.upper() == consts.RECOVER_REBOOT:
+            # default to hard reboot if operation_params was not specified
+            if not isinstance(op_params, dict):
+                op_params = {}
+            if consts.REBOOT_TYPE not in op_params.keys():
+                op_params[consts.REBOOT_TYPE] = consts.REBOOT_HARD
+
+        if operation.upper() == consts.RECOVER_RECREATE:
+            # recreate is implemented in base class
+            return super(ServerProfile, self).do_recover(obj, **options)
+        else:
+            method = getattr(self, "handle_" + operation.lower())
+            return method(obj, **op_params)
 
     def handle_reboot(self, obj, **options):
         """Handler for the reboot operation."""
@@ -1582,9 +1739,9 @@ class ServerProfile(base.Profile):
             return None, False
 
         server_id = obj.physical_id
-        reboot_type = options.get(self.REBOOT_TYPE, self.REBOOT_SOFT)
-        if (not isinstance(reboot_type, six.string_types) or
-                reboot_type not in self.REBOOT_TYPES):
+        reboot_type = options.get(consts.REBOOT_TYPE, consts.REBOOT_SOFT)
+        if (not isinstance(reboot_type, str) or
+                reboot_type.upper() not in consts.REBOOT_TYPES):
             return server_id, False
 
         nova_driver = self.compute(obj)
@@ -1599,7 +1756,7 @@ class ServerProfile(base.Profile):
         except exc.InternalError as ex:
             raise exc.EResourceOperation(op='rebooting', type='server',
                                          id=server_id,
-                                         message=six.text_type(ex))
+                                         message=str(ex))
 
     def handle_rebuild(self, obj, **options):
         """Handler for the rebuild operation.
@@ -1622,7 +1779,7 @@ class ServerProfile(base.Profile):
         except exc.InternalError as ex:
             raise exc.EResourceOperation(op='rebuilding', type='server',
                                          id=server_id,
-                                         message=six.text_type(ex))
+                                         message=str(ex))
 
         if server is None:
             return None, False
@@ -1640,12 +1797,12 @@ class ServerProfile(base.Profile):
         except exc.InternalError as ex:
             raise exc.EResourceOperation(op='rebuilding', type='server',
                                          id=server_id,
-                                         message=six.text_type(ex))
+                                         message=str(ex))
 
     def handle_change_password(self, obj, **options):
         """Handler for the change_password operation."""
         password = options.get(self.ADMIN_PASSWORD, None)
-        if (password is None or not isinstance(password, six.string_types)):
+        if (password is None or not isinstance(password, str)):
             return False
         return self._handle_generic_op(obj, 'server_change_password',
                                        'change_password',
