@@ -10,8 +10,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
-import six
+from unittest import mock
+
 from webob import exc
 
 from oslo_serialization import jsonutils
@@ -130,7 +130,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.index, req)
 
-        self.assertEqual("Invalid parameter balrog", six.text_type(ex))
+        self.assertEqual("Invalid parameter balrog", str(ex))
         self.assertFalse(mock_parse.called)
         self.assertFalse(mock_call.called)
 
@@ -146,7 +146,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.index, req)
 
-        self.assertEqual("bad param", six.text_type(ex))
+        self.assertEqual("bad param", str(ex))
         mock_parse.assert_called_once_with(
             'ReceiverListRequest', req,
             {
@@ -168,7 +168,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.index, req)
 
-        self.assertEqual("bad param", six.text_type(ex))
+        self.assertEqual("bad param", str(ex))
         mock_parse.assert_called_once_with(
             'ReceiverListRequest', req,
             {
@@ -190,7 +190,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.index, req)
 
-        self.assertEqual("bad param", six.text_type(ex))
+        self.assertEqual("bad param", str(ex))
         mock_parse.assert_called_once_with(
             'ReceiverListRequest', req,
             {
@@ -213,7 +213,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.index, req)
 
-        self.assertEqual("bad param", six.text_type(ex))
+        self.assertEqual("bad param", str(ex))
         mock_parse.assert_called_once_with(
             'ReceiverListRequest', req,
             {
@@ -250,7 +250,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.index, req)
         self.assertEqual(403, resp.status_int)
-        self.assertIn('403 Forbidden', six.text_type(resp))
+        self.assertIn('403 Forbidden', str(resp))
 
     @mock.patch.object(util, 'parse_request')
     @mock.patch.object(rpc_client.EngineClient, 'call')
@@ -318,7 +318,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
                                self.controller.create,
                                req, body=body)
 
-        self.assertEqual("bad param", six.text_type(ex))
+        self.assertEqual("bad param", str(ex))
         mock_parse.assert_called_once_with(
             'ReceiverCreateRequest', req, body, 'receiver')
         self.assertFalse(mock_call.called)
@@ -344,7 +344,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
                                self.controller.create,
                                req, body=body)
 
-        self.assertEqual("miss type", six.text_type(ex))
+        self.assertEqual("miss type", str(ex))
         mock_parse.assert_called_once_with(
             'ReceiverCreateRequest', req, body, 'receiver')
         self.assertFalse(mock_call.called)
@@ -371,7 +371,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
                                self.controller.create,
                                req, body=body)
 
-        self.assertEqual("bad type", six.text_type(ex))
+        self.assertEqual("bad type", str(ex))
         mock_parse.assert_called_once_with(
             'ReceiverCreateRequest', req, body, 'receiver')
         self.assertFalse(mock_call.called)
@@ -397,7 +397,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
                                self.controller.create,
                                req, body=body)
 
-        self.assertEqual("bad action", six.text_type(ex))
+        self.assertEqual("bad action", str(ex))
         mock_parse.assert_called_once_with(
             'ReceiverCreateRequest', req, body, 'receiver')
         self.assertFalse(mock_call.called)
@@ -468,7 +468,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
                                               req, receiver_id=wid)
 
         self.assertEqual(403, resp.status_int)
-        self.assertIn('403 Forbidden', six.text_type(resp))
+        self.assertIn('403 Forbidden', str(resp))
 
     @mock.patch.object(util, 'parse_request')
     @mock.patch.object(rpc_client.EngineClient, 'call')
@@ -524,7 +524,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
                                req, receiver_id=wid, body=body)
 
         self.assertEqual("Malformed request data, missing 'receiver' key "
-                         "in request body.", six.text_type(ex))
+                         "in request body.", str(ex))
         self.assertFalse(mock_parse.called)
         self.assertFalse(mock_call.called)
 
@@ -573,7 +573,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
                                self.controller.update,
                                req, receiver_id=wid, body=body)
 
-        self.assertEqual("bad param", six.text_type(ex))
+        self.assertEqual("bad param", str(ex))
         mock_parse.assert_called_once_with(
             'ReceiverUpdateRequest', req, mock.ANY)
         self.assertFalse(mock_call.called)
@@ -619,7 +619,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
                                               body=body)
 
         self.assertEqual(403, resp.status_int)
-        self.assertIn('403 Forbidden', six.text_type(resp))
+        self.assertIn('403 Forbidden', str(resp))
 
     @mock.patch.object(util, 'parse_request')
     @mock.patch.object(rpc_client.EngineClient, 'call')
@@ -652,7 +652,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.delete, req,
                                receiver_id=wid)
-        self.assertEqual("bad identity", six.text_type(ex))
+        self.assertEqual("bad identity", str(ex))
         mock_parse.assert_called_once_with(
             'ReceiverDeleteRequest', req, {'identity': wid})
         self.assertFalse(mock_call.called)
@@ -685,7 +685,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
                                               req, receiver_id=wid)
 
         self.assertEqual(403, resp.status_int)
-        self.assertIn('403 Forbidden', six.text_type(resp))
+        self.assertIn('403 Forbidden', str(resp))
 
     @mock.patch.object(util, 'parse_request')
     @mock.patch.object(rpc_client.EngineClient, 'call')
@@ -719,7 +719,7 @@ class ReceiverControllerTest(shared.ControllerTest, base.SenlinTestCase):
         ex = self.assertRaises(exc.HTTPBadRequest,
                                self.controller.notify, req,
                                receiver_id=wid)
-        self.assertEqual("bad identity", six.text_type(ex))
+        self.assertEqual("bad identity", str(ex))
         mock_parse.assert_called_once_with(
             'ReceiverNotifyRequest', req, {'identity': wid})
         self.assertFalse(mock_call.called)
