@@ -10,10 +10,10 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from unittest import mock
+
 from oslo_db.sqlalchemy import utils as sa_utils
 from oslo_utils import timeutils as tu
-import six
 
 from senlin.common import consts
 from senlin.common import exception
@@ -60,6 +60,9 @@ class DBAPIProfileTest(base.SenlinTestCase):
         res = db_api.profile_get(admin_ctx, profile.id, project_safe=True)
         self.assertIsNotNone(res)
 
+        res = db_api.profile_get(admin_ctx, profile.id, project_safe=False)
+        self.assertIsNotNone(res)
+
     def test_profile_get_not_found(self):
         profile = db_api.profile_get(self.ctx, 'BogusProfileID')
         self.assertIsNone(profile)
@@ -82,6 +85,12 @@ class DBAPIProfileTest(base.SenlinTestCase):
         retobj = db_api.profile_get_by_name(self.ctx, 'non-exist')
         self.assertIsNone(retobj)
 
+        # duplicated name
+        shared.create_profile(self.ctx, name=profile_name)
+        self.assertRaises(exception.MultipleChoices,
+                          db_api.profile_get_by_name,
+                          self.ctx, profile_name)
+
     def test_profile_get_by_name_diff_project(self):
         profile_name = 'my_best_profile'
         shared.create_profile(self.ctx, name=profile_name)
@@ -179,7 +188,8 @@ class DBAPIProfileTest(base.SenlinTestCase):
         admin_ctx = utils.dummy_context(project='a-different-project',
                                         is_admin=True)
         profiles = db_api.profile_get_all(admin_ctx, project_safe=True)
-        self.assertEqual(0, len(profiles))
+        self.assertEqual(2, len(profiles))
+
         profiles = db_api.profile_get_all(admin_ctx, project_safe=False)
         self.assertEqual(2, len(profiles))
 
@@ -340,8 +350,8 @@ class DBAPIProfileTest(base.SenlinTestCase):
         profile_id = profile.id
         ex = self.assertRaises(exception.EResourceBusy,
                                db_api.profile_delete, self.ctx, profile_id)
-        self.assertEqual('The profile (%s) is busy now.' % profile_id,
-                         six.text_type(ex))
+        self.assertEqual("The profile '%s' is busy now." % profile_id,
+                         str(ex))
 
         db_api.cluster_delete(self.ctx, cluster.id)
         db_api.profile_delete(self.ctx, profile_id)
@@ -353,8 +363,8 @@ class DBAPIProfileTest(base.SenlinTestCase):
         profile_id = profile.id
         ex = self.assertRaises(exception.EResourceBusy,
                                db_api.profile_delete, self.ctx, profile_id)
-        self.assertEqual('The profile (%s) is busy now.' % profile_id,
-                         six.text_type(ex))
+        self.assertEqual("The profile '%s' is busy now." % profile_id,
+                         str(ex))
 
         db_api.node_delete(self.ctx, node.id)
         db_api.profile_delete(self.ctx, profile_id)
