@@ -10,7 +10,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 from oslo_utils import timeutils
 
@@ -102,7 +103,7 @@ class SortParamTest(base.SenlinTestCase):
 class ServiceAliveTest(base.SenlinTestCase):
 
     def test_alive(self):
-        cfg.CONF.set_override('periodic_interval', 100, enforce_type=True)
+        cfg.CONF.set_override('periodic_interval', 100)
         service = mock.Mock(updated_at=timeutils.utcnow())
 
         res = utils.is_service_dead(service)
@@ -110,7 +111,7 @@ class ServiceAliveTest(base.SenlinTestCase):
         self.assertFalse(res)
 
     def test_dead(self):
-        cfg.CONF.set_override('periodic_interval', 0, enforce_type=True)
+        cfg.CONF.set_override('periodic_interval', 0)
         service = mock.Mock(updated_at=timeutils.utcnow())
 
         res = utils.is_service_dead(service)
