@@ -11,12 +11,12 @@
 # under the License.
 
 import copy
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 
-from senlin.drivers.openstack import keystone_v3 as kv3
-from senlin.drivers.openstack import sdk
+from senlin.drivers.os import keystone_v3 as kv3
+from senlin.drivers import sdk
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
@@ -55,7 +55,7 @@ class TestKeystoneV3(base.SenlinTestCase):
         mock_create.return_value = self.conn
         kc = kv3.KeystoneClient({'k': 'v'})
 
-        # no trustee/projec filter, matching 1st
+        # no trustee/project filter, matching 1st
         res = kc.trust_get_by_trustor('USER_A')
         self.assertEqual(trust1, res)
 
@@ -116,6 +116,45 @@ class TestKeystoneV3(base.SenlinTestCase):
             allow_redelegation=True, roles=[])
         self.conn.reset_mock()
 
+    def test_trust_create_conf_roles(self, mock_create):
+        cfg.CONF.set_override('trust_roles', ['r1', 'r2'])
+        self.conn.identity.create_trust.return_value = 'new_trust'
+        mock_create.return_value = self.conn
+        kc = kv3.KeystoneClient({'k': 'v'})
+
+        res = kc.trust_create('ID_JOHN', 'ID_DOE', 'PROJECT_ID', [
+            'r1', 'r2', 'r3'])
+
+        self.assertEqual('new_trust', res)
+        self.conn.identity.create_trust.assert_called_once_with(
+            trustor_user_id='ID_JOHN', trustee_user_id='ID_DOE',
+            project_id='PROJECT_ID', impersonation=True,
+            allow_redelegation=True, roles=[{'name': 'r1'}, {'name': 'r2'}])
+        self.conn.reset_mock()
+
+        cfg.CONF.set_override('trust_roles', [])
+        res = kc.trust_create('ID_JOHN', 'ID_DOE', 'PROJECT_ID',
+                              ['r1', 'r2'])
+
+        self.assertEqual('new_trust', res)
+        self.conn.identity.create_trust.assert_called_once_with(
+            trustor_user_id='ID_JOHN', trustee_user_id='ID_DOE',
+            project_id='PROJECT_ID', impersonation=True,
+            allow_redelegation=True,
+            roles=[{'name': 'r1'}, {'name': 'r2'}])
+        self.conn.reset_mock()
+
+        # impersonation
+        res = kc.trust_create('ID_JOHN', 'ID_DOE', 'PROJECT_ID',
+                              impersonation=False)
+
+        self.assertEqual('new_trust', res)
+        self.conn.identity.create_trust.assert_called_once_with(
+            trustor_user_id='ID_JOHN', trustee_user_id='ID_DOE',
+            project_id='PROJECT_ID', impersonation=False,
+            allow_redelegation=True, roles=[])
+        self.conn.reset_mock()
+
     @mock.patch.object(sdk, 'authenticate')
     def test_get_token(self, mock_auth, mock_create):
         access_info = {'token': '123', 'user_id': 'abc', 'project_id': 'xyz'}
@@ -137,18 +176,17 @@ class TestKeystoneV3(base.SenlinTestCase):
         self.assertEqual('abc', user_id)
 
     def test_get_service_credentials(self, mock_create):
-        cfg.CONF.set_override('auth_url', 'FAKE_URL', group='authentication',
-                              enforce_type=True)
+        cfg.CONF.set_override('auth_url', 'FAKE_URL', group='authentication')
         cfg.CONF.set_override('service_username', 'FAKE_USERNAME',
-                              group='authentication', enforce_type=True)
+                              group='authentication')
         cfg.CONF.set_override('service_password', 'FAKE_PASSWORD',
-                              group='authentication', enforce_type=True)
+                              group='authentication')
         cfg.CONF.set_override('service_project_name', 'FAKE_PROJECT',
-                              group='authentication', enforce_type=True)
+                              group='authentication')
         cfg.CONF.set_override('service_user_domain', 'FAKE_DOMAIN_1',
-                              group='authentication', enforce_type=True)
+                              group='authentication')
         cfg.CONF.set_override('service_project_domain', 'FAKE_DOMAIN_2',
-                              group='authentication', enforce_type=True)
+                              group='authentication')
         expected = {
             'auth_url': 'FAKE_URL',
             'username': 'FAKE_USERNAME',
@@ -190,8 +228,7 @@ class TestKeystoneV3(base.SenlinTestCase):
         self.assertEqual([], res)
 
     def test_get_senlin_endpoint(self, mock_create):
-        cfg.CONF.set_override('default_region_name', 'RegionN',
-                              enforce_type=True)
+        cfg.CONF.set_override('default_region_name', 'RegionN')
         self.conn.session.get_endpoint.return_value = 'http://web.com:1234/v1'
         mock_create.return_value = self.conn
         kc = kv3.KeystoneClient({'k': 'v'})
@@ -200,4 +237,5 @@ class TestKeystoneV3(base.SenlinTestCase):
 
         self.assertEqual('http://web.com:1234/v1', res)
         self.conn.session.get_endpoint.assert_called_once_with(
-            service_type='clustering', interface='public', region='RegionN')
+            service_type='clustering', interface='public',
+            region_name='RegionN')
