@@ -11,7 +11,7 @@
 # under the License.
 
 import eventlet
-import mock
+from unittest import mock
 
 from oslo_context import context as oslo_context
 
@@ -21,6 +21,7 @@ from senlin.drivers.os import lbaas
 from senlin.drivers.os import neutron_v2
 from senlin.drivers.os import octavia_v2
 from senlin.engine import node as nodem
+from senlin.profiles import base as pb
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
@@ -62,6 +63,7 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
             "url_path": "/index.html",
             "expected_codes": "200,201,202"
         }
+        self.availability_zone = 'my_fake_az'
 
     def test_init(self):
         conn_params = self.context.to_dict()
@@ -126,7 +128,7 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
         self.assertFalse(res)
         mock_sleep.assert_called_once_with(10)
 
-    def test_lb_create_succeeded(self):
+    def test_lb_create_succeeded_subnet(self):
         lb_obj = mock.Mock()
         listener_obj = mock.Mock()
         pool_obj = mock.Mock()
@@ -149,11 +151,13 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
 
         self.lb_driver._wait_for_lb_ready = mock.Mock()
         self.lb_driver._wait_for_lb_ready.return_value = True
-        status, res = self.lb_driver.lb_create(self.vip, self.pool, self.hm)
+        status, res = self.lb_driver.lb_create(self.vip, self.pool, self.hm,
+                                               self.availability_zone)
 
         self.assertTrue(status)
         self.oc.loadbalancer_create.assert_called_once_with(
-            'SUBNET_ID', self.vip['address'], self.vip['admin_state_up'])
+            'SUBNET_ID', None, self.vip['address'], self.vip['admin_state_up'],
+            availability_zone=self.availability_zone)
         self.assertEqual('LB_ID', res['loadbalancer'])
         self.assertEqual('192.168.1.100', res['vip_address'])
         self.oc.listener_create.assert_called_once_with(
@@ -175,6 +179,64 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
         self.lb_driver._wait_for_lb_ready.assert_has_calls(
             calls, any_order=False)
 
+    def test_lb_create_succeeded_network(self):
+        vip = {
+            'network': 'network-01',
+            'address': '192.168.1.100',
+            'admin_state_up': True,
+            'protocol': 'HTTP',
+            'protocol_port': 80,
+            'connection_limit': 50
+        }
+        lb_obj = mock.Mock()
+        listener_obj = mock.Mock()
+        pool_obj = mock.Mock()
+        hm_obj = mock.Mock()
+        lb_obj.id = 'LB_ID'
+        lb_obj.vip_address = '192.168.1.100'
+        listener_obj.id = 'LISTENER_ID'
+        pool_obj.id = 'POOL_ID'
+        network_obj = mock.Mock()
+        network_obj.name = 'network'
+        network_obj.id = 'NETWORK_ID'
+        hm_obj.id = 'HEALTHMONITOR_ID'
+
+        self.oc.loadbalancer_create.return_value = lb_obj
+        self.oc.listener_create.return_value = listener_obj
+        self.oc.pool_create.return_value = pool_obj
+        self.oc.healthmonitor_create.return_value = hm_obj
+        self.nc.network_get.return_value = network_obj
+
+        self.lb_driver._wait_for_lb_ready = mock.Mock()
+        self.lb_driver._wait_for_lb_ready.return_value = True
+        status, res = self.lb_driver.lb_create(vip, self.pool, self.hm,
+                                               self.availability_zone)
+
+        self.assertTrue(status)
+        self.oc.loadbalancer_create.assert_called_once_with(
+            None, 'NETWORK_ID', vip['address'], vip['admin_state_up'],
+            availability_zone=self.availability_zone)
+        self.assertEqual('LB_ID', res['loadbalancer'])
+        self.assertEqual('192.168.1.100', res['vip_address'])
+        self.oc.listener_create.assert_called_once_with(
+            'LB_ID', vip['protocol'], vip['protocol_port'],
+            vip['connection_limit'], vip['admin_state_up'])
+        self.assertEqual('LISTENER_ID', res['listener'])
+        self.oc.pool_create.assert_called_once_with(
+            self.pool['lb_method'], 'LISTENER_ID', self.pool['protocol'],
+            self.pool['admin_state_up'])
+        self.assertEqual('POOL_ID', res['pool'])
+        self.oc.healthmonitor_create.assert_called_once_with(
+            self.hm['type'], self.hm['delay'], self.hm['timeout'],
+            self.hm['max_retries'], 'POOL_ID', self.hm['admin_state_up'],
+            self.hm['http_method'], self.hm['url_path'],
+            self.hm['expected_codes'])
+        self.assertEqual('HEALTHMONITOR_ID', res['healthmonitor'])
+        self.lb_driver._wait_for_lb_ready.assert_called_with('LB_ID')
+        calls = [mock.call('LB_ID') for i in range(1, 5)]
+        self.lb_driver._wait_for_lb_ready.assert_has_calls(
+            calls, any_order=False)
+
     def test_lb_create_loadbalancer_creation_failed(self):
         lb_obj = mock.Mock()
         lb_obj.id = 'LB_ID'
@@ -194,7 +256,8 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
         msg = _('Failed in creating loadbalancer (%s).') % 'LB_ID'
         self.assertEqual(msg, res)
         self.oc.loadbalancer_create.assert_called_once_with(
-            'SUBNET_ID', self.vip['address'], self.vip['admin_state_up'])
+            'SUBNET_ID', None, self.vip['address'], self.vip['admin_state_up'],
+            availability_zone=None)
         self.lb_driver._wait_for_lb_ready.assert_called_once_with('LB_ID')
         self.lb_driver.lb_delete.assert_called_once_with(
             loadbalancer='LB_ID')
@@ -239,7 +302,8 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
         msg = _('Failed in creating listener (%s).') % 'LISTENER_ID'
         self.assertEqual(msg, res)
         self.oc.loadbalancer_create.assert_called_once_with(
-            'SUBNET_ID', self.vip['address'], self.vip['admin_state_up'])
+            'SUBNET_ID', None, self.vip['address'], self.vip['admin_state_up'],
+            availability_zone=None)
         self.oc.listener_create.assert_called_once_with(
             'LB_ID', self.vip['protocol'], self.vip['protocol_port'],
             self.vip['connection_limit'], self.vip['admin_state_up'])
@@ -283,7 +347,8 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
         msg = _('Failed in creating pool (%s).') % 'POOL_ID'
         self.assertEqual(msg, res)
         self.oc.loadbalancer_create.assert_called_once_with(
-            'SUBNET_ID', self.vip['address'], self.vip['admin_state_up'])
+            'SUBNET_ID', None, self.vip['address'], self.vip['admin_state_up'],
+            availability_zone=None)
         self.oc.listener_create.assert_called_once_with(
             'LB_ID', self.vip['protocol'], self.vip['protocol_port'],
             self.vip['connection_limit'], self.vip['admin_state_up'])
@@ -434,9 +499,9 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
         self.lb_driver._wait_for_lb_ready.assert_called_once_with(
             'LB_ID', ignore_not_found=True)
 
-    @mock.patch.object(nodem.Node, 'load')
+    @mock.patch.object(pb.Profile, 'load')
     @mock.patch.object(oslo_context, 'get_current')
-    def test_member_add_succeeded(self, mock_get_current, mock_load):
+    def test_member_add_succeeded(self, mock_get_current, mock_pb_load):
         fake_context = mock.Mock()
         mock_get_current.return_value = fake_context
         node = mock.Mock()
@@ -445,6 +510,7 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
         port = '80'
         subnet = 'subnet'
         subnet_obj = mock.Mock(id='SUBNET_ID', network_id='NETWORK_ID')
+        subnet_obj.ip_version = '4'
         subnet_obj.name = 'subnet'
         network_obj = mock.Mock(id='NETWORK_ID')
         network_obj.name = 'network1'
@@ -452,12 +518,12 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
         node_detail = {
             'name': 'node-01',
             'addresses': {
-                'network1': [{'addr': 'ipaddr_net1'}],
-                'network2': [{'addr': 'ipaddr_net2'}]
+                'network1': [{'addr': 'ipaddr1_net1', 'version': '6'},
+                             {'addr': 'ipaddr2_net1', 'version': '4'}],
+                'network2': [{'addr': 'ipaddr_net2', 'version': '4'}]
             }
         }
-        mock_load.return_value = node
-        node.get_details.return_value = node_detail
+        mock_pb_load.return_value.do_get_details.return_value = node_detail
 
         self.nc.subnet_get.return_value = subnet_obj
         self.nc.network_get.return_value = network_obj
@@ -469,11 +535,11 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
         self.assertEqual('MEMBER_ID', res)
         self.nc.subnet_get.assert_called_once_with(subnet)
         self.nc.network_get.assert_called_once_with('NETWORK_ID')
+        # Make sure the ip matches with subnet ip_version
         self.oc.pool_member_create.assert_called_once_with(
-            pool_id, 'ipaddr_net1', port, 'SUBNET_ID')
+            pool_id, 'ipaddr2_net1', port, 'SUBNET_ID')
         self.lb_driver._wait_for_lb_ready.assert_has_calls(
             [mock.call('LB_ID'), mock.call('LB_ID')])
-        mock_load.assert_called_once_with(fake_context, db_node=node)
 
     @mock.patch.object(oslo_context, 'get_current')
     def test_member_add_subnet_get_failed(self, mock_get_current):
@@ -498,24 +564,26 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
                                         'subnet')
         self.assertIsNone(res)
 
+    @mock.patch.object(pb.Profile, 'load')
     @mock.patch.object(nodem.Node, 'load')
     @mock.patch.object(oslo_context, 'get_current')
     def test_member_add_lb_unready_for_member_create(self, mock_get_current,
-                                                     mock_load):
+                                                     mock_load, mock_pb_load):
         node = mock.Mock()
         subnet_obj = mock.Mock(id='SUBNET_ID', network_id='NETWORK_ID')
         subnet_obj.name = 'subnet'
+        subnet_obj.ip_version = '4'
         network_obj = mock.Mock(id='NETWORK_ID')
         network_obj.name = 'network1'
         node_detail = {
             'name': 'node-01',
             'addresses': {
-                'network1': [{'addr': 'ipaddr_net1'}],
-                'network2': [{'addr': 'ipaddr_net2'}]
+                'network1': [{'addr': 'ipaddr_net1', 'version': '4'}],
+                'network2': [{'addr': 'ipaddr_net2', 'version': '4'}]
             }
         }
         mock_load.return_value = node
-        node.get_details.return_value = node_detail
+        mock_pb_load.return_value.do_get_details.return_value = node_detail
 
         # Exception happens in pool_member_create
         self.lb_driver._wait_for_lb_ready = mock.Mock()
@@ -529,24 +597,26 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
         self.assertIsNone(res)
         self.lb_driver._wait_for_lb_ready.assert_called_once_with('LB_ID')
 
+    @mock.patch.object(pb.Profile, 'load')
     @mock.patch.object(nodem.Node, 'load')
     @mock.patch.object(oslo_context, 'get_current')
     def test_member_add_member_create_failed(self, mock_get_current,
-                                             mock_load):
+                                             mock_load, mock_pb_load):
         node = mock.Mock()
         subnet_obj = mock.Mock(id='SUBNET_ID', network_id='NETWORK_ID')
         subnet_obj.name = 'subnet'
+        subnet_obj.ip_version = '4'
         network_obj = mock.Mock(id='NETWORK_ID')
         network_obj.name = 'network1'
         node_detail = {
             'name': 'node-01',
             'addresses': {
-                'network1': [{'addr': 'ipaddr_net1'}],
-                'network2': [{'addr': 'ipaddr_net2'}]
+                'network1': [{'addr': 'ipaddr_net1', 'version': '4'}],
+                'network2': [{'addr': 'ipaddr_net2', 'version': '4'}]
             }
         }
         mock_load.return_value = node
-        node.get_details.return_value = node_detail
+        mock_pb_load.return_value.do_get_details.return_value = node_detail
 
         # Exception happens in pool_member_create
         self.lb_driver._wait_for_lb_ready = mock.Mock()
@@ -559,23 +629,57 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
                                         'subnet')
         self.assertIsNone(res)
 
+    @mock.patch.object(pb.Profile, 'load')
     @mock.patch.object(nodem.Node, 'load')
     @mock.patch.object(oslo_context, 'get_current')
-    def test_member_add_wait_for_lb_timeout(self, mock_get_current, mock_load):
+    def test_member_add_ip_version_match_failed(self, mock_get_current,
+                                                mock_load, mock_pb_load):
         node = mock.Mock()
         subnet_obj = mock.Mock(id='SUBNET_ID', network_id='NETWORK_ID')
         subnet_obj.name = 'subnet'
+        subnet_obj.ip_version = '4'
         network_obj = mock.Mock(id='NETWORK_ID')
         network_obj.name = 'network1'
         node_detail = {
             'name': 'node-01',
             'addresses': {
-                'network1': [{'addr': 'ipaddr_net1'}],
-                'network2': [{'addr': 'ipaddr_net2'}]
+                'network1': [{'addr': 'ipaddr_net1', 'version': '6'}],
+                'network2': [{'addr': 'ipaddr_net2', 'version': '6'}]
             }
         }
         mock_load.return_value = node
-        node.get_details.return_value = node_detail
+        mock_pb_load.return_value.do_get_details.return_value = node_detail
+
+        # Node does not match with subnet ip_version
+        self.lb_driver._wait_for_lb_ready = mock.Mock()
+        self.lb_driver._wait_for_lb_ready.return_value = True
+        self.nc.subnet_get.return_value = subnet_obj
+        self.nc.network_get.return_value = network_obj
+        self.oc.pool_member_create = mock.Mock(id='MEMBER_ID')
+        res = self.lb_driver.member_add(node, 'LB_ID', 'POOL_ID', 80,
+                                        'subnet')
+        self.assertIsNone(res)
+
+    @mock.patch.object(pb.Profile, 'load')
+    @mock.patch.object(nodem.Node, 'load')
+    @mock.patch.object(oslo_context, 'get_current')
+    def test_member_add_wait_for_lb_timeout(self, mock_get_current, mock_load,
+                                            mock_pb_load):
+        node = mock.Mock()
+        subnet_obj = mock.Mock(id='SUBNET_ID', network_id='NETWORK_ID')
+        subnet_obj.name = 'subnet'
+        subnet_obj.ip_version = '4'
+        network_obj = mock.Mock(id='NETWORK_ID')
+        network_obj.name = 'network1'
+        node_detail = {
+            'name': 'node-01',
+            'addresses': {
+                'network1': [{'addr': 'ipaddr_net1', 'version': '4'}],
+                'network2': [{'addr': 'ipaddr_net2', 'version': '4'}]
+            }
+        }
+        mock_load.return_value = node
+        mock_pb_load.return_value.do_get_details.return_value = node_detail
 
         # Wait for lb ready timeout after creating member
         self.lb_driver._wait_for_lb_ready = mock.Mock()
@@ -586,9 +690,11 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
                                         'subnet')
         self.assertIsNone(res)
 
+    @mock.patch.object(pb.Profile, 'load')
     @mock.patch.object(nodem.Node, 'load')
     @mock.patch.object(oslo_context, 'get_current')
-    def test_member_add_node_not_in_subnet(self, mock_get_current, mock_load):
+    def test_member_add_node_not_in_subnet(self, mock_get_current, mock_load,
+                                           mock_pb_load):
         node = mock.Mock()
         lb_id = 'LB_ID'
         pool_id = 'POOL_ID'
@@ -604,7 +710,7 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
             }
         }
         mock_load.return_value = node
-        node.get_details.return_value = node_detail
+        mock_pb_load.return_value.do_get_details.return_value = node_detail
 
         self.nc.network_get.return_value = network_obj
         self.lb_driver._wait_for_lb_ready = mock.Mock()
@@ -625,7 +731,8 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
         self.assertTrue(res)
         self.oc.pool_member_delete.assert_called_once_with(pool_id, member_id)
         self.lb_driver._wait_for_lb_ready.assert_has_calls(
-            [mock.call(lb_id), mock.call(lb_id)])
+            [mock.call(lb_id, ignore_not_found=True),
+             mock.call(lb_id, ignore_not_found=True)])
 
     def test_member_remove_lb_unready_for_member_delete(self):
         self.lb_driver._wait_for_lb_ready = mock.Mock()
@@ -633,7 +740,9 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
 
         res = self.lb_driver.member_remove('LB_ID', 'POOL_ID', 'MEMBER_ID')
         self.assertFalse(res)
-        self.lb_driver._wait_for_lb_ready.assert_called_once_with('LB_ID')
+        self.lb_driver._wait_for_lb_ready.assert_has_calls(
+            [mock.call('LB_ID', ignore_not_found=True),
+             mock.call('LB_ID', ignore_not_found=True)])
 
     def test_member_remove_member_delete_failed(self):
         self.lb_driver._wait_for_lb_ready = mock.Mock()
@@ -654,4 +763,5 @@ class TestOctaviaLBaaSDriver(base.SenlinTestCase):
         res = self.lb_driver.member_remove('LB_ID', 'POOL_ID', 'MEMBER_ID')
         self.assertIsNone(res)
         self.lb_driver._wait_for_lb_ready.assert_has_calls(
-            [mock.call('LB_ID'), mock.call('LB_ID')])
+            [mock.call('LB_ID', ignore_not_found=True),
+             mock.call('LB_ID', ignore_not_found=True)])
