@@ -10,11 +10,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from openstack import exceptions as sdk_exc
+from unittest import mock
+
 from oslo_config import cfg
 
-from senlin.drivers.openstack import nova_v2
-from senlin.drivers.openstack import sdk
+from senlin.drivers.os import nova_v2
+from senlin.drivers import sdk
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
@@ -50,18 +52,23 @@ class TestNovaV2(base.SenlinTestCase):
         d.flavor_find('foo', False)
         self.compute.find_flavor.assert_called_once_with('foo', False)
 
-    def test_image_find(self):
+    def test_keypair_create(self):
         d = nova_v2.NovaClient(self.conn_params)
-        d.image_find('foo')
-        self.compute.find_image.assert_called_once_with('foo', False)
-        self.compute.find_image.reset_mock()
+        d.keypair_create(name='foo')
+        self.compute.create_keypair.assert_called_once_with(name='foo')
 
-        d.image_find('foo', True)
-        self.compute.find_image.assert_called_once_with('foo', True)
-        self.compute.find_image.reset_mock()
+    def test_keypair_delete(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        d.keypair_delete('foo')
+        self.compute.delete_keypair.assert_called_once_with('foo', False)
+        self.compute.delete_keypair.reset_mock()
 
-        d.image_find('foo', False)
-        self.compute.find_image.assert_called_once_with('foo', False)
+        d.keypair_delete('foo', True)
+        self.compute.delete_keypair.assert_called_once_with('foo', True)
+        self.compute.delete_keypair.reset_mock()
+
+        d.keypair_delete('foo', False)
+        self.compute.delete_keypair.assert_called_once_with('foo', False)
 
     def test_keypair_find(self):
         d = nova_v2.NovaClient(self.conn_params)
@@ -128,17 +135,17 @@ class TestNovaV2(base.SenlinTestCase):
 
         res = d.server_resize_confirm('fakeid')
 
-        self.assertEqual(d.conn.compute.confirm_resize_server.return_value,
+        self.assertEqual(d.conn.compute.confirm_server_resize.return_value,
                          res)
-        d.conn.compute.confirm_resize_server.assert_called_once_with('fakeid')
+        d.conn.compute.confirm_server_resize.assert_called_once_with('fakeid')
 
     def test_server_resize_revert(self):
         d = nova_v2.NovaClient(self.conn_params)
 
         res = d.server_resize_revert('fakeid')
 
-        self.assertEqual(d.conn.compute.revert_resize_server.return_value, res)
-        d.conn.compute.revert_resize_server.assert_called_once_with('fakeid')
+        self.assertEqual(d.conn.compute.revert_server_resize.return_value, res)
+        d.conn.compute.revert_server_resize.assert_called_once_with('fakeid')
 
     def test_server_reboot(self):
         d = nova_v2.NovaClient(self.conn_params)
@@ -158,6 +165,160 @@ class TestNovaV2(base.SenlinTestCase):
         self.assertEqual(target.return_value, res)
         target.assert_called_once_with('fakeid', 'new_password')
 
+    def test_server_pause(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.pause_server
+
+        res = d.server_pause(server)
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server)
+
+    def test_server_unpause(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.unpause_server
+
+        res = d.server_unpause(server)
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server)
+
+    def test_server_suspend(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.suspend_server
+
+        res = d.server_suspend(server)
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server)
+
+    def test_server_resume(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.resume_server
+
+        res = d.server_resume(server)
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server)
+
+    def test_server_lock(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.lock_server
+
+        res = d.server_lock(server)
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server)
+
+    def test_server_unlock(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.unlock_server
+
+        res = d.server_unlock(server)
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server)
+
+    def test_server_start(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.start_server
+
+        res = d.server_start(server)
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server)
+
+    def test_server_stop(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.stop_server
+
+        res = d.server_stop(server)
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server)
+
+    def test_server_rescue(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.rescue_server
+
+        res = d.server_rescue(server)
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server, admin_pass=None, image_ref=None)
+
+    def test_server_rescue_with_params(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.rescue_server
+
+        res = d.server_rescue(server, admin_pass='PASS', image_ref='IMAGE')
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server, admin_pass='PASS',
+                                       image_ref='IMAGE')
+
+    def test_server_unrescue(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.unrescue_server
+
+        res = d.server_unrescue(server)
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server)
+
+    def test_server_migrate(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.migrate_server
+
+        res = d.server_migrate(server)
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server)
+
+    def test_server_evacuate(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.evacuate_server
+
+        res = d.server_evacuate(server)
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server, host=None, admin_pass=None,
+                                       force=None)
+
+    def test_server_evacuate_with_params(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.evacuate_server
+
+        res = d.server_evacuate(server, host='HOST', admin_pass='PASS',
+                                force='True')
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server, host='HOST', admin_pass='PASS',
+                                       force='True')
+
+    def test_server_create_image(self):
+        d = nova_v2.NovaClient(self.conn_params)
+        server = mock.Mock()
+        target = d.conn.compute.create_server_image
+
+        res = d.server_create_image(server, 'snapshot', metadata='meta')
+
+        self.assertEqual(target.return_value, res)
+        target.assert_called_once_with(server, 'snapshot', 'meta')
+
     def test_wait_for_server(self):
         self.compute.find_server.return_value = 'foo'
 
@@ -176,7 +337,7 @@ class TestNovaV2(base.SenlinTestCase):
 
     def test_wait_for_server_with_default_timeout(self):
         self.compute.find_server.return_value = 'foo'
-        timeout = cfg.CONF.default_action_timeout
+        timeout = cfg.CONF.default_nova_timeout
 
         d = nova_v2.NovaClient(self.conn_params)
         d.wait_for_server('foo')
@@ -193,7 +354,7 @@ class TestNovaV2(base.SenlinTestCase):
         self.compute.wait_for_delete.assert_called_once_with('FOO', wait=120)
 
     def test_wait_for_server_delete_with_default_timeout(self):
-        cfg.CONF.set_override('default_action_timeout', 360, enforce_type=True)
+        cfg.CONF.set_override('default_nova_timeout', 360)
         self.compute.find_server.return_value = 'FOO'
 
         d = nova_v2.NovaClient(self.conn_params)
@@ -211,9 +372,9 @@ class TestNovaV2(base.SenlinTestCase):
     def test_server_interface_create(self):
         server = mock.Mock()
         d = nova_v2.NovaClient(self.conn_params)
-        d.server_interface_create(server, name='foo')
+        d.server_interface_create(server, port_id='fake-port-id')
         self.compute.create_server_interface.assert_called_once_with(
-            server, name='foo')
+            server, port_id='fake-port-id')
 
     def test_server_interface_list(self):
         d = nova_v2.NovaClient(self.conn_params)
@@ -243,6 +404,22 @@ class TestNovaV2(base.SenlinTestCase):
         self.compute.delete_server_interface.assert_called_once_with(
             'foo', server, True)
 
+    def test_server_floatingip_associate(self):
+        server = mock.Mock()
+        d = nova_v2.NovaClient(self.conn_params)
+        d.server_floatingip_associate(server, 'fake_floatingip')
+        self.compute.add_floating_ip_to_server.assert_called_once_with(
+            server, 'fake_floatingip'
+        )
+
+    def test_server_floatingip_disassociate(self):
+        server = mock.Mock()
+        d = nova_v2.NovaClient(self.conn_params)
+        d.server_floatingip_disassociate(server, 'fake_floatingip')
+        self.compute.remove_floating_ip_from_server.assert_called_once_with(
+            server, 'fake_floatingip'
+        )
+
     def test_server_metadata_get(self):
         server = mock.Mock()
         res_server = mock.Mock()
@@ -260,16 +437,42 @@ class TestNovaV2(base.SenlinTestCase):
         server = mock.Mock()
         res_server = mock.Mock()
         res_server.metadata = {
-            'k1': 'v1'
+            'k1': 'v1',
+            'k2': 'v2'
         }
         self.compute.get_server_metadata.return_value = res_server
 
         d = nova_v2.NovaClient(self.conn_params)
-        d.server_metadata_update(server, {'k2': 'v2'})
-        self.compute.delete_server_metadata.assert_called_once_with(
-            server, ['k1'])
-        self.compute.set_server_metadata.assert_called_once_with(
-            server, k2='v2')
+        d.server_metadata_update(server, {'k3': 'v3', 'k4': 'v4'})
+        self.compute.set_server_metadata.assert_has_calls(
+            [mock.call(server, k3='v3'), mock.call(server, k4='v4')],
+            any_order=True)
+        self.compute.delete_server_metadata.assert_has_calls(
+            [mock.call(server, ['k1']), mock.call(server, ['k2'])],
+            any_order=True)
+
+    def test_server_metadata_update_forbidden(self):
+        server = mock.Mock()
+        res_server = mock.Mock()
+        res_server.metadata = {
+            'k1': 'v1',
+            'forbidden_key': 'forbidden_data',
+            'k2': 'v2'
+        }
+        self.compute.get_server_metadata.return_value = res_server
+        self.compute.delete_server_metadata.side_effect = [
+            None, sdk_exc.HttpException(http_status=403), None]
+        self.compute.set_server_metadata.side_effect = [
+            None, sdk_exc.HttpException(http_status=403), None]
+
+        d = nova_v2.NovaClient(self.conn_params)
+        d.server_metadata_update(server, {'k3': 'v3', 'k4': 'v4', 'k5': 'v5'})
+        self.compute.set_server_metadata.assert_has_calls(
+            [mock.call(server, k3='v3'), mock.call(server, k4='v4'),
+             mock.call(server, k5='v5')], any_order=True)
+        self.compute.delete_server_metadata.assert_has_calls(
+            [mock.call(server, ['k1']), mock.call(server, ['forbidden_key']),
+             mock.call(server, ['k2'])], any_order=True)
 
     def test_server_metadata_delete(self):
         server = mock.Mock()
@@ -281,12 +484,16 @@ class TestNovaV2(base.SenlinTestCase):
     def test_validate_azs(self):
         nc = nova_v2.NovaClient(self.conn_params)
 
-        fake_azs = [
-            {'zoneState': {'available': 1}, 'zoneName': 'AZ1'},
-            {'zoneState': {'available': 1}, 'zoneName': 'AZ2'},
-            {'zoneState': {'available': 1}, 'zoneName': 'AZ3'},
-            {'zoneState': {'available': 1}, 'zoneName': 'AZ4'}
-        ]
+        az1 = mock.Mock()
+        az1.name = 'AZ1'
+        az1.state = {'available': True}
+        az2 = mock.Mock()
+        az2.name = 'AZ2'
+        az2.state = {'available': True}
+        az3 = mock.Mock()
+        az3.name = 'AZ3'
+        az3.state = {'available': True}
+        fake_azs = [az1, az2, az3]
 
         self.patchobject(nc, 'availability_zone_list', return_value=fake_azs)
 
@@ -357,3 +564,17 @@ class TestNovaV2(base.SenlinTestCase):
         d.service_force_down(service)
         self.compute.force_service_down.assert_called_once_with(
             service, service.host, service.binary)
+
+    def test_create_volume_attachment(self):
+        server = mock.Mock()
+        d = nova_v2.NovaClient(self.conn_params)
+        kwargs = {
+            'serverId': server,
+            'volumeId': 'fake_volume',
+        }
+        d.create_volume_attachment(server, **kwargs)
+
+    def test_delete_volume_attachment(self):
+        server = mock.Mock()
+        d = nova_v2.NovaClient(self.conn_params)
+        d.delete_volume_attachment('fake_volume', server, ignore_missing=True)
