@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from unittest import mock
 
 from senlin.common import consts
 from senlin.engine.actions import base as ab
@@ -19,8 +19,10 @@ from senlin.engine import cluster as cm
 from senlin.engine import dispatcher
 from senlin.engine.notifications import message as msg
 from senlin.objects import action as ao
+from senlin.objects import cluster_policy as cpo
 from senlin.objects import dependency as dobj
 from senlin.objects import node as no
+from senlin.objects import receiver as ro
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
@@ -37,8 +39,8 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(dobj.Dependency, 'create')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__delete_nodes_single(self, mock_wait, mock_start, mock_dep,
-                                  mock_action, mock_update, mock_load):
+    def test_delete_nodes_single(self, mock_wait, mock_start, mock_dep,
+                                 mock_action, mock_update, mock_load):
         # prepare mocks
         cluster = mock.Mock(id='FAKE_CLUSTER', desired_capacity=100, config={})
 
@@ -47,6 +49,7 @@ class ClusterDeleteTest(base.SenlinTestCase):
         action = ca.ClusterAction(cluster.id, 'CLUSTER_ACTION', self.ctx)
         action.id = 'CLUSTER_ACTION_ID'
         action.inputs = {'destroy_after_deletion': False}
+        action.context = self.ctx
         mock_wait.return_value = (action.RES_OK, 'All dependents completed')
         mock_action.return_value = 'NODE_ACTION_ID'
 
@@ -58,11 +61,13 @@ class ClusterDeleteTest(base.SenlinTestCase):
         self.assertEqual('All dependents completed', res_msg)
         mock_action.assert_called_once_with(
             action.context, 'NODE_ID', 'NODE_DELETE',
-            name='node_delete_NODE_ID', cause='Derived Action', inputs={})
+            name='node_delete_NODE_ID', cause='Derived Action',
+            cluster_id='FAKE_CLUSTER', inputs={})
         mock_dep.assert_called_once_with(action.context, ['NODE_ACTION_ID'],
                                          'CLUSTER_ACTION_ID')
-        mock_update.assert_called_once_with(action.context, 'NODE_ACTION_ID',
-                                            {'status': 'READY'})
+        mock_update.assert_called_with(action.context,
+                                       'NODE_ACTION_ID',
+                                       {'status': 'READY'})
         mock_start.assert_called_once_with()
         mock_wait.assert_called_once_with()
         self.assertEqual(['NODE_ID'], action.outputs['nodes_removed'])
@@ -73,9 +78,9 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(dobj.Dependency, 'create')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__delete_nodes_single_stop_node(self, mock_wait, mock_start,
-                                            mock_dep, mock_action, mock_update,
-                                            mock_load):
+    def test_delete_nodes_single_stop_node(self, mock_wait, mock_start,
+                                           mock_dep, mock_action, mock_update,
+                                           mock_load):
         # prepare mocks
         cluster = mock.Mock(id='FAKE_CLUSTER', desired_capacity=100,
                             config={'cluster.stop_node_before_delete': True})
@@ -98,9 +103,12 @@ class ClusterDeleteTest(base.SenlinTestCase):
             mock.call(action.context, 'NODE_ID', 'NODE_OPERATION',
                       name='node_delete_NODE_ID',
                       cause='Derived Action',
-                      inputs={'operation': 'stop'}),
+                      cluster_id='FAKE_CLUSTER',
+                      inputs={'operation': 'stop',
+                              'update_parent_status': False}),
             mock.call(action.context, 'NODE_ID', 'NODE_DELETE',
                       name='node_delete_NODE_ID',
+                      cluster_id='FAKE_CLUSTER',
                       cause='Derived Action', inputs={})
         ]
         mock_action.assert_has_calls(create_actions)
@@ -124,8 +132,8 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(dobj.Dependency, 'create')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__delete_nodes_multi(self, mock_wait, mock_start, mock_dep,
-                                 mock_action, mock_update, mock_load):
+    def test_delete_nodes_multi(self, mock_wait, mock_start, mock_dep,
+                                mock_action, mock_update, mock_load):
         # prepare mocks
         cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=100, config={})
         mock_load.return_value = cluster
@@ -157,7 +165,7 @@ class ClusterDeleteTest(base.SenlinTestCase):
         cluster.remove_node.assert_has_calls([
             mock.call('NODE_1'), mock.call('NODE_2')])
 
-    def test__delete_empty(self, mock_load):
+    def test_delete_empty(self, mock_load):
         # prepare mocks
         cluster = mock.Mock(id='CLUSTER_ID')
         mock_load.return_value = cluster
@@ -176,8 +184,8 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(dobj.Dependency, 'create')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__delete_nodes_with_pd(self, mock_wait, mock_start, mock_dep,
-                                   mock_action, mock_update, mock_load):
+    def test_delete_nodes_with_pd(self, mock_wait, mock_start, mock_dep,
+                                  mock_action, mock_update, mock_load):
         # prepare mocks
         cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=100, config={})
         mock_load.return_value = cluster
@@ -200,7 +208,8 @@ class ClusterDeleteTest(base.SenlinTestCase):
         self.assertEqual('All dependents completed', res_msg)
         mock_action.assert_called_once_with(
             action.context, 'NODE_ID', 'NODE_LEAVE',
-            name='node_delete_NODE_ID', cause='Derived Action', inputs={})
+            name='node_delete_NODE_ID', cluster_id='CLUSTER_ID',
+            cause='Derived Action', inputs={})
 
     @mock.patch.object(ao.Action, 'update')
     @mock.patch.object(ab.Action, 'create')
@@ -209,11 +218,11 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(msg.Message, 'post_lifecycle_hook_message')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__delete_nodes_with_lifecycle_hook(self, mock_wait, mock_start,
-                                               mock_post, mock_dep,
-                                               mock_node_get,
-                                               mock_action, mock_update,
-                                               mock_load):
+    def test_delete_nodes_with_lifecycle_hook(self, mock_wait, mock_start,
+                                              mock_post, mock_dep,
+                                              mock_node_get,
+                                              mock_action, mock_update,
+                                              mock_load):
         # prepare mocks
         cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=100, config={})
         mock_load.return_value = cluster
@@ -229,6 +238,7 @@ class ClusterDeleteTest(base.SenlinTestCase):
                 }
             }
         }
+        action.owner = 'OWNER_ID'
         mock_wait.return_value = (action.RES_OK, 'All dependents completed')
         mock_action.return_value = 'NODE_ACTION_ID'
         mock_node_get.return_value = mock.Mock(
@@ -245,10 +255,12 @@ class ClusterDeleteTest(base.SenlinTestCase):
             action.context, 'NODE_ID', 'NODE_DELETE',
             name='node_delete_NODE_ID',
             cause='Derived Action with Lifecycle Hook',
+            cluster_id='CLUSTER_ID',
             inputs={})
         update_calls = [
             mock.call(action.context, 'NODE_ACTION_ID',
-                      {'status': 'WAITING_LIFECYCLE_COMPLETION'}),
+                      {'status': 'WAITING_LIFECYCLE_COMPLETION',
+                       'owner': 'OWNER_ID'}),
         ]
         mock_update.assert_has_calls(update_calls)
         mock_post.assert_called_once_with('NODE_ACTION_ID', 'NODE_ID',
@@ -264,7 +276,7 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(msg.Message, 'post_lifecycle_hook_message')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__delete_nodes_with_lifecycle_hook_failed_node(
+    def test_delete_nodes_with_lifecycle_hook_failed_node(
             self, mock_wait, mock_start, mock_post, mock_dep, mock_node_get,
             mock_action, mock_update, mock_load):
         self.delete_nodes_with_lifecycle_hook_invalid_node(
@@ -279,7 +291,7 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(msg.Message, 'post_lifecycle_hook_message')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__delete_nodes_with_lifecycle_hook_missing_node(
+    def test_delete_nodes_with_lifecycle_hook_missing_node(
             self, mock_wait, mock_start, mock_post, mock_dep, mock_node_get,
             mock_action, mock_update, mock_load):
         self.delete_nodes_with_lifecycle_hook_invalid_node(
@@ -304,6 +316,7 @@ class ClusterDeleteTest(base.SenlinTestCase):
                 }
             }
         }
+        action.owner = None
         mock_wait.return_value = (action.RES_OK, 'All dependents completed')
         mock_action.return_value = 'NODE_ACTION_ID'
         mock_node_get.return_value = mock_node_obj
@@ -317,10 +330,12 @@ class ClusterDeleteTest(base.SenlinTestCase):
         mock_action.assert_called_once_with(
             action.context, 'NODE_ID', 'NODE_DELETE',
             name='node_delete_NODE_ID',
+            cluster_id='CLUSTER_ID',
             cause='Derived Action with Lifecycle Hook', inputs={})
         update_calls = [
             mock.call(action.context, 'NODE_ACTION_ID',
-                      {'status': 'READY'}),
+                      {'status': 'READY',
+                       'owner': None}),
         ]
         mock_update.assert_has_calls(update_calls)
         mock_post.assert_not_called()
@@ -335,14 +350,14 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(msg.Message, 'post_lifecycle_hook_message')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__delete_nodes_with_lifecycle_hook_timeout(self, mock_wait,
-                                                       mock_start,
-                                                       mock_post, mock_dep,
-                                                       mock_node_get,
-                                                       mock_action,
-                                                       mock_update,
-                                                       mock_check_status,
-                                                       mock_load):
+    def test_delete_nodes_with_lifecycle_hook_timeout(self, mock_wait,
+                                                      mock_start,
+                                                      mock_post, mock_dep,
+                                                      mock_node_get,
+                                                      mock_action,
+                                                      mock_update,
+                                                      mock_check_status,
+                                                      mock_load):
         # prepare mocks
         cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=100, config={})
         mock_load.return_value = cluster
@@ -358,6 +373,7 @@ class ClusterDeleteTest(base.SenlinTestCase):
                 }
             }
         }
+        action.owner = 'OWNER_ID'
         mock_wait.side_effect = [
             (action.RES_LIFECYCLE_HOOK_TIMEOUT, 'Timeout'),
             (action.RES_OK, 'All dependents completed')
@@ -376,12 +392,15 @@ class ClusterDeleteTest(base.SenlinTestCase):
         mock_action.assert_called_once_with(
             action.context, 'NODE_ID', 'NODE_DELETE',
             name='node_delete_NODE_ID',
+            cluster_id='CLUSTER_ID',
             cause='Derived Action with Lifecycle Hook', inputs={})
         update_calls = [
             mock.call(action.context, 'NODE_ACTION_ID',
-                      {'status': 'WAITING_LIFECYCLE_COMPLETION'}),
+                      {'status': 'WAITING_LIFECYCLE_COMPLETION',
+                       'owner': 'OWNER_ID'}),
             mock.call(action.context, 'NODE_ACTION_ID',
-                      {'status': 'READY'}),
+                      {'status': 'READY',
+                       'owner': None}),
         ]
         mock_update.assert_has_calls(update_calls)
         mock_post.assert_called_once_with('NODE_ACTION_ID', 'NODE_ID',
@@ -395,9 +414,9 @@ class ClusterDeleteTest(base.SenlinTestCase):
         mock_wait.assert_has_calls(wait_calls)
 
     @mock.patch.object(ab.Action, 'create')
-    def test__delete_nodes_with_lifecycle_hook_invalid_type(self,
-                                                            mock_action,
-                                                            mock_load):
+    def test_delete_nodes_with_lifecycle_hook_invalid_type(self,
+                                                           mock_action,
+                                                           mock_load):
         # prepare mocks
         cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=100, config={})
         mock_load.return_value = cluster
@@ -424,10 +443,10 @@ class ClusterDeleteTest(base.SenlinTestCase):
 
     @mock.patch.object(ao.Action, 'update')
     @mock.patch.object(ab.Action, 'create')
-    def test__delete_nodes_with_lifecycle_hook_unsupported_webhook(self,
-                                                                   mock_action,
-                                                                   mock_update,
-                                                                   mock_load):
+    def test_delete_nodes_with_lifecycle_hook_unsupported_webhook(self,
+                                                                  mock_action,
+                                                                  mock_update,
+                                                                  mock_load):
         # prepare mocks
         cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=100, config={})
         mock_load.return_value = cluster
@@ -453,8 +472,8 @@ class ClusterDeleteTest(base.SenlinTestCase):
                          "'webhook' is not implemented", res_msg)
 
     @mock.patch.object(ca.ClusterAction, '_remove_nodes_normally')
-    def test__delete_nodes_failed_remove_stop_node(self, mock_remove,
-                                                   mock_load):
+    def test_delete_nodes_failed_remove_stop_node(self, mock_remove,
+                                                  mock_load):
         # prepare mocks
         cluster = mock.Mock(id='ID',
                             config={'cluster.stop_node_before_delete': True})
@@ -474,14 +493,15 @@ class ClusterDeleteTest(base.SenlinTestCase):
         self.assertEqual(action.RES_OK, res_code)
         self.assertEqual({}, action.data)
         remove_calls = [
-            mock.call('NODE_OPERATION', ['NODE_ID'], {'operation': 'stop'}),
+            mock.call('NODE_OPERATION', ['NODE_ID'],
+                      {'operation': 'stop', 'update_parent_status': False}),
             mock.call('NODE_DELETE', ['NODE_ID']),
         ]
         mock_remove.assert_has_calls(remove_calls)
 
     @mock.patch.object(ca.ClusterAction, '_remove_nodes_with_hook')
     @mock.patch.object(ca.ClusterAction, '_remove_nodes_normally')
-    def test__delete_nodes_with_lifecycle_hook_failed_remove_stop_node(
+    def test_delete_nodes_with_lifecycle_hook_failed_remove_stop_node(
             self, mock_remove_normally, mock_remove_hook, mock_load):
         # prepare mocks
         cluster = mock.Mock(id='ID',
@@ -511,12 +531,11 @@ class ClusterDeleteTest(base.SenlinTestCase):
         self.assertEqual(action.RES_OK, res_code)
         mock_remove_hook.assert_called_once_with(
             'NODE_OPERATION', ['NODE_ID'], lifecycle_hook,
-            {'operation': 'stop'})
+            {'operation': 'stop', 'update_parent_status': False})
         mock_remove_normally.assert_called_once_with('NODE_DELETE',
                                                      ['NODE_ID'])
 
-    @mock.patch.object(ao.Action, 'delete_by_target')
-    def test_do_delete_success(self, mock_action, mock_load):
+    def test_do_delete_success(self, mock_load):
         node1 = mock.Mock(id='NODE_1')
         node2 = mock.Mock(id='NODE_2')
         cluster = mock.Mock(id='FAKE_CLUSTER', nodes=[node1, node2],
@@ -541,10 +560,96 @@ class ClusterDeleteTest(base.SenlinTestCase):
                                                    'Deletion in progress.')
         mock_delete.assert_called_once_with(['NODE_1', 'NODE_2'])
         cluster.do_delete.assert_called_once_with(action.context)
-        mock_action.assert_called_once_with(
-            action.context, 'FAKE_CLUSTER',
-            action_excluded=['CLUSTER_DELETE'],
-            status=['SUCCEEDED', 'FAILED'])
+
+    @mock.patch.object(ro.Receiver, 'get_all')
+    @mock.patch.object(cpo.ClusterPolicy, 'get_all')
+    def test_do_delete_with_policies(self, mock_policies,
+                                     mock_receivers, mock_load):
+        mock_policy1 = mock.Mock()
+        mock_policy1.policy_id = 'POLICY_ID1'
+        mock_policy2 = mock.Mock()
+        mock_policy2.policy_id = 'POLICY_ID2'
+
+        mock_policies.return_value = [mock_policy1, mock_policy2]
+        mock_receivers.return_value = []
+
+        node1 = mock.Mock(id='NODE_1')
+        node2 = mock.Mock(id='NODE_2')
+        cluster = mock.Mock(id='FAKE_CLUSTER', nodes=[node1, node2],
+                            DELETING='DELETING')
+        cluster.do_delete.return_value = True
+        mock_load.return_value = cluster
+
+        cluster.detach_policy = mock.Mock()
+        cluster.detach_policy.return_value = (True, 'OK')
+
+        action = ca.ClusterAction(cluster.id, 'CLUSTER_DELETE', self.ctx)
+        action.data = {}
+
+        mock_delete = self.patchobject(action, '_delete_nodes',
+                                       return_value=(action.RES_OK, 'Good'))
+
+        # do it
+        res_code, res_msg = action.do_delete()
+
+        self.assertEqual(action.RES_OK, res_code)
+        self.assertEqual('Good', res_msg)
+        self.assertEqual({'deletion': {'destroy_after_deletion': True}},
+                         action.data)
+        cluster.set_status.assert_called_once_with(action.context, 'DELETING',
+                                                   'Deletion in progress.')
+        mock_delete.assert_called_once_with(['NODE_1', 'NODE_2'])
+        cluster.do_delete.assert_called_once_with(action.context)
+        detach_calls = [mock.call(action.context, 'POLICY_ID1'),
+                        mock.call(action.context, 'POLICY_ID2')]
+        cluster.detach_policy.assert_has_calls(detach_calls)
+
+    @mock.patch.object(ro.Receiver, 'delete')
+    @mock.patch.object(ro.Receiver, 'get_all')
+    @mock.patch.object(cpo.ClusterPolicy, 'get_all')
+    def test_do_delete_with_receivers(self, mock_policies,
+                                      mock_receivers, mock_rec_delete,
+                                      mock_load):
+        mock_receiver1 = mock.Mock()
+        mock_receiver1.id = 'RECEIVER_ID1'
+        mock_receiver2 = mock.Mock()
+        mock_receiver2.id = 'RECEIVER_ID2'
+
+        mock_policies.return_value = []
+        mock_receivers.return_value = [mock_receiver1, mock_receiver2]
+
+        node1 = mock.Mock(id='NODE_1')
+        node2 = mock.Mock(id='NODE_2')
+        cluster = mock.Mock(id='FAKE_CLUSTER', nodes=[node1, node2],
+                            DELETING='DELETING')
+        cluster.do_delete.return_value = True
+        mock_load.return_value = cluster
+
+        cluster.detach_policy = mock.Mock()
+        cluster.detach_policy.return_value = (True, 'OK')
+
+        action = ca.ClusterAction(cluster.id, 'CLUSTER_DELETE', self.ctx)
+        action.data = {}
+
+        mock_delete = self.patchobject(action, '_delete_nodes',
+                                       return_value=(action.RES_OK, 'Good'))
+
+        # do it
+        res_code, res_msg = action.do_delete()
+
+        self.assertEqual(action.RES_OK, res_code)
+        self.assertEqual('Good', res_msg)
+        self.assertEqual({'deletion': {'destroy_after_deletion': True}},
+                         action.data)
+        cluster.set_status.assert_called_once_with(action.context, 'DELETING',
+                                                   'Deletion in progress.')
+        mock_delete.assert_called_once_with(['NODE_1', 'NODE_2'])
+        cluster.do_delete.assert_called_once_with(action.context)
+
+        cluster.detach_policy.assert_not_called()
+        rec_delete_calls = [mock.call(action.context, 'RECEIVER_ID1'),
+                            mock.call(action.context, 'RECEIVER_ID2')]
+        mock_rec_delete.assert_has_calls(rec_delete_calls)
 
     def test_do_delete_failed_delete_nodes_timeout(self, mock_load):
         node = mock.Mock(id='NODE_1')
@@ -773,8 +878,8 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(dobj.Dependency, 'create')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__remove_nodes_normally(self, mock_wait, mock_start, mock_dep,
-                                    mock_action, mock_update, mock_load):
+    def test_remove_nodes_normally(self, mock_wait, mock_start, mock_dep,
+                                   mock_action, mock_update, mock_load):
         # prepare mocks
         cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=100)
         mock_load.return_value = cluster
@@ -810,9 +915,9 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(msg.Message, 'post_lifecycle_hook_message')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__remove_nodes_with_hook(self, mock_wait, mock_start, mock_post,
-                                     mock_dep, mock_node_get, mock_action,
-                                     mock_update, mock_load):
+    def test_remove_nodes_with_hook(self, mock_wait, mock_start, mock_post,
+                                    mock_dep, mock_node_get, mock_action,
+                                    mock_update, mock_load):
         # prepare mocks
         cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=100)
         mock_load.return_value = cluster
@@ -828,6 +933,7 @@ class ClusterDeleteTest(base.SenlinTestCase):
                 }
             }
         }
+        action.owner = 'OWNER_ID'
         mock_wait.return_value = (action.RES_OK, 'All dependents completed')
         mock_action.return_value = 'NODE_ACTION_ID'
         mock_node_get.return_value = mock.Mock(
@@ -842,11 +948,12 @@ class ClusterDeleteTest(base.SenlinTestCase):
         self.assertEqual(1, mock_dep.call_count)
         mock_action.assert_called_once_with(
             action.context, 'NODE_ID', 'NODE_DELETE',
-            name='node_delete_NODE_ID',
+            name='node_delete_NODE_ID', cluster_id='CLUSTER_ID',
             cause='Derived Action with Lifecycle Hook', inputs={})
         update_calls = [
             mock.call(action.context, 'NODE_ACTION_ID',
-                      {'status': 'WAITING_LIFECYCLE_COMPLETION'}),
+                      {'status': 'WAITING_LIFECYCLE_COMPLETION',
+                       'owner': 'OWNER_ID'}),
         ]
         mock_update.assert_has_calls(update_calls)
         mock_post.assert_called_once_with('NODE_ACTION_ID', 'NODE_ID',
@@ -860,9 +967,9 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(dobj.Dependency, 'create')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__remove_nodes_normally_failed_wait(self, mock_wait, mock_start,
-                                                mock_dep, mock_action,
-                                                mock_update, mock_load):
+    def test_remove_nodes_normally_failed_wait(self, mock_wait, mock_start,
+                                               mock_dep, mock_action,
+                                               mock_update, mock_load):
         # prepare mocks
         cluster = mock.Mock(id='ID', config={})
         mock_load.return_value = cluster
@@ -888,9 +995,9 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(dobj.Dependency, 'create')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__remove_nodes_hook_failed_wait(self, mock_wait, mock_start,
-                                            mock_dep, mock_action,
-                                            mock_update, mock_load):
+    def test_remove_nodes_hook_failed_wait(self, mock_wait, mock_start,
+                                           mock_dep, mock_action,
+                                           mock_update, mock_load):
         # prepare mocks
         cluster = mock.Mock(id='ID', config={})
         mock_load.return_value = cluster
@@ -925,10 +1032,10 @@ class ClusterDeleteTest(base.SenlinTestCase):
     @mock.patch.object(msg.Message, 'post_lifecycle_hook_message')
     @mock.patch.object(dispatcher, 'start_action')
     @mock.patch.object(ca.ClusterAction, '_wait_for_dependents')
-    def test__delete_nodes_with_error_nodes(self, mock_wait, mock_start,
-                                            mock_post, mock_dep,
-                                            mock_node_get, mock_action,
-                                            mock_update, mock_load):
+    def test_delete_nodes_with_error_nodes(self, mock_wait, mock_start,
+                                           mock_post, mock_dep,
+                                           mock_node_get, mock_action,
+                                           mock_update, mock_load):
         # prepare mocks
         cluster = mock.Mock(id='CLUSTER_ID', desired_capacity=100)
         mock_load.return_value = cluster
@@ -945,12 +1052,13 @@ class ClusterDeleteTest(base.SenlinTestCase):
                 }
             }
         }
+        action.owner = 'OWNER_ID'
         mock_action.side_effect = ['NODE_ACTION_1', 'NODE_ACTION_2']
         mock_wait.return_value = (action.RES_OK, 'All dependents completed')
         node1 = mock.Mock(status=consts.NS_ACTIVE, id='NODE_1',
-                          physical_id="nova-server-1")
-        node2 = mock.Mock(status=consts.NS_ACTIVE, id='NODE_2',
                           physical_id=None)
+        node2 = mock.Mock(status=consts.NS_ACTIVE, id='NODE_2',
+                          physical_id="nova-server-1")
         mock_node_get.side_effect = [node1, node2]
         # do it
         res_code, res_msg = action._remove_nodes_with_hook(
@@ -960,23 +1068,27 @@ class ClusterDeleteTest(base.SenlinTestCase):
         self.assertEqual(action.RES_OK, res_code)
         self.assertEqual('All dependents completed', res_msg)
         update_calls = [
-            mock.call(action.context, 'NODE_ACTION_1',
-                      {'status': 'WAITING_LIFECYCLE_COMPLETION'}),
-            mock.call(action.context, 'NODE_ACTION_2', {'status': 'READY'})
+            mock.call(action.context, 'NODE_ACTION_1', {'status': 'READY',
+                                                        'owner': None}),
+            mock.call(action.context, 'NODE_ACTION_2',
+                      {'status': 'WAITING_LIFECYCLE_COMPLETION',
+                       'owner': 'OWNER_ID'})
         ]
         mock_update.assert_has_calls(update_calls)
         create_actions = [
             mock.call(action.context, 'NODE_1', 'NODE_DELETE',
                       name='node_delete_NODE_1',
+                      cluster_id='CLUSTER_ID',
                       cause='Derived Action with Lifecycle Hook', inputs={}),
             mock.call(action.context, 'NODE_2', 'NODE_DELETE',
                       name='node_delete_NODE_2',
+                      cluster_id='CLUSTER_ID',
                       cause='Derived Action with Lifecycle Hook', inputs={})
         ]
         mock_action.assert_has_calls(create_actions)
 
-        mock_post.assert_called_once_with('NODE_ACTION_1', 'NODE_1',
-                                          node1.physical_id,
+        mock_post.assert_called_once_with('NODE_ACTION_2', 'NODE_2',
+                                          node2.physical_id,
                                           consts.LIFECYCLE_NODE_TERMINATION)
         mock_start.assert_called_once_with()
         mock_wait.assert_called_once_with(action.data['hooks']['timeout'])
