@@ -10,11 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 from oslo_context import context as oslo_ctx
 from oslo_utils import timeutils
-import six
 
 from senlin.common import context
 from senlin.common import exception
@@ -55,9 +55,9 @@ class TestReceiver(base.SenlinTestCase):
             'type': 'webhook',
             'cluster_id': CLUSTER_ID,
             'action': 'test-action',
-            'user': self.context.user,
-            'project': self.context.project,
-            'domain': self.context.domain,
+            'user': self.context.user_id,
+            'project': self.context.project_id,
+            'domain': self.context.domain_id,
             'created_at': timeutils.utcnow(True),
             'updated_at': None,
             'actor': self.actor,
@@ -121,7 +121,7 @@ class TestReceiver(base.SenlinTestCase):
     def test_receiver_store(self):
         receiver = rb.Receiver('webhook', CLUSTER_ID, 'test-action',
                                name='test_receiver_123456',
-                               project=self.context.project)
+                               project=self.context.project_id)
         self.assertIsNone(receiver.id)
 
         receiver_id = receiver.store(self.context)
@@ -162,9 +162,9 @@ class TestReceiver(base.SenlinTestCase):
                                       'FAKE_ACTION',
                                       name='test_receiver_2234')
 
-        self.assertEqual(ctx.user, receiver.user)
-        self.assertEqual(ctx.project, receiver.project)
-        self.assertEqual(ctx.domain, receiver.domain)
+        self.assertEqual(ctx.user_id, receiver.user)
+        self.assertEqual(ctx.project_id, receiver.project)
+        self.assertEqual(ctx.domain_id, receiver.domain)
         self.assertEqual('123abc', receiver.actor['trust_id'])
         mock_c_get.assert_called_once_with(ctx, 'user1', 'project1')
 
@@ -177,9 +177,9 @@ class TestReceiver(base.SenlinTestCase):
                                       'FAKE_ACTION',
                                       name='test_receiver_2234')
 
-        self.assertEqual(ctx.user, receiver.user)
-        self.assertEqual(ctx.project, receiver.project)
-        self.assertEqual(ctx.domain, receiver.domain)
+        self.assertEqual(ctx.user_id, receiver.user)
+        self.assertEqual(ctx.project_id, receiver.project)
+        self.assertEqual(ctx.domain_id, receiver.domain)
         self.assertIsNone(receiver.actor['trust_id'])
 
     @mock.patch.object(rm.Message, 'initialize_channel')
@@ -187,9 +187,9 @@ class TestReceiver(base.SenlinTestCase):
         receiver = rb.Receiver.create(self.context, 'message', None,
                                       None, name='test_receiver_2234')
 
-        self.assertEqual(self.context.user, receiver.user)
-        self.assertEqual(self.context.project, receiver.project)
-        self.assertEqual(self.context.domain, receiver.domain)
+        self.assertEqual(self.context.user_id, receiver.user)
+        self.assertEqual(self.context.project_id, receiver.project)
+        self.assertEqual(self.context.domain_id, receiver.domain)
 
     def _verify_receiver(self, receiver, result):
         self.assertEqual(receiver.id, result.id)
@@ -222,8 +222,8 @@ class TestReceiver(base.SenlinTestCase):
         ex = self.assertRaises(exception.ResourceNotFound,
                                rb.Receiver.load,
                                self.context, 'fake-receiver', None)
-        self.assertEqual('The receiver (fake-receiver) could not be found.',
-                         six.text_type(ex))
+        self.assertEqual("The receiver 'fake-receiver' could not "
+                         "be found.", str(ex))
 
     def test_receiver_load_diff_project(self):
         receiver = self._create_receiver('receiver-1', UUID1)
@@ -232,36 +232,13 @@ class TestReceiver(base.SenlinTestCase):
         ex = self.assertRaises(exception.ResourceNotFound,
                                rb.Receiver.load,
                                new_context, UUID1, None)
-        self.assertEqual('The receiver (%s) could not be found.' % UUID1,
-                         six.text_type(ex))
+        self.assertEqual("The receiver '%s' could not be found." % UUID1,
+                         str(ex))
 
         res = rb.Receiver.load(new_context, receiver.id, project_safe=False)
         self.assertIsNotNone(res)
         self.assertEqual(receiver.id, res.id)
 
-    def test_receiver_load_all(self):
-        result = rb.Receiver.load_all(self.context)
-        self.assertEqual([], [w for w in result])
-
-        receiver1 = self._create_receiver('receiver-1', UUID1)
-        receiver2 = self._create_receiver('receiver-2', UUID2)
-
-        result = rb.Receiver.load_all(self.context)
-        receivers = [w for w in result]
-        self.assertEqual(2, len(receivers))
-        self.assertEqual(receiver1.id, receivers[0].id)
-        self.assertEqual(receiver2.id, receivers[1].id)
-
-    def test_receiver_load_all_diff_project(self):
-        self._create_receiver('receiver-1', UUID1)
-        self._create_receiver('receiver-2', UUID2)
-
-        new_context = utils.dummy_context(project='a-different-project')
-        result = rb.Receiver.load_all(new_context)
-        self.assertEqual(0, len(list(result)))
-        result = rb.Receiver.load_all(new_context, project_safe=False)
-        self.assertEqual(2, len(list(result)))
-
     def test_receiver_to_dict(self):
         receiver = self._create_receiver('test-receiver', UUID1)
         self.assertIsNotNone(receiver.id)
@@ -310,18 +287,17 @@ class TestReceiver(base.SenlinTestCase):
         mock_receiver.release_channel.assert_called_once_with(self.context)
         mock_delete.assert_called_once_with(self.context, 'test-receiver-id')
 
-    @mock.patch.object(context, "get_service_context")
+    @mock.patch.object(context, "get_service_credentials")
     @mock.patch.object(driver_base, "SenlinDriver")
-    def test__get_base_url_succeeded(self, mock_senlin_driver,
-                                     mock_get_service_context):
-        cfg.CONF.set_override('default_region_name', 'RegionOne',
-                              enforce_type=True)
+    def test_get_base_url_succeeded(self, mock_senlin_driver,
+                                    mock_get_service_creds):
+        cfg.CONF.set_override('default_region_name', 'RegionOne')
         fake_driver = mock.Mock()
         fake_kc = mock.Mock()
         fake_cred = mock.Mock()
         mock_senlin_driver.return_value = fake_driver
         fake_driver.identity.return_value = fake_kc
-        mock_get_service_context.return_value = fake_cred
+        mock_get_service_creds.return_value = fake_cred
 
         fake_kc.get_senlin_endpoint.return_value = "http://web.com:1234/v1"
 
@@ -331,21 +307,20 @@ class TestReceiver(base.SenlinTestCase):
 
         res = receiver._get_base_url()
         self.assertEqual("http://web.com:1234/v1", res)
-        mock_get_service_context.assert_called_once_with()
+        mock_get_service_creds.assert_called_once_with()
         fake_kc.get_senlin_endpoint.assert_called_once_with()
 
-    @mock.patch.object(context, "get_service_context")
+    @mock.patch.object(context, "get_service_credentials")
     @mock.patch.object(driver_base, "SenlinDriver")
-    def test__get_base_url_failed_get_endpoint_exception(
-            self, mock_senlin_driver, mock_get_service_context):
-        cfg.CONF.set_override('default_region_name', 'RegionOne',
-                              enforce_type=True)
+    def test_get_base_url_failed_get_endpoint_exception(
+            self, mock_senlin_driver, mock_get_service_creds):
+        cfg.CONF.set_override('default_region_name', 'RegionOne')
         fake_driver = mock.Mock()
         fake_kc = mock.Mock()
         fake_cred = mock.Mock()
         mock_senlin_driver.return_value = fake_driver
         fake_driver.identity.return_value = fake_kc
-        mock_get_service_context.return_value = fake_cred
+        mock_get_service_creds.return_value = fake_cred
 
         fake_kc.get_senlin_endpoint.side_effect = exception.InternalError(
             message='Error!')
@@ -356,13 +331,13 @@ class TestReceiver(base.SenlinTestCase):
 
         res = receiver._get_base_url()
         self.assertIsNone(res)
-        mock_get_service_context.assert_called_once_with()
+        mock_get_service_creds.assert_called_once_with()
         fake_kc.get_senlin_endpoint.assert_called_once_with()
 
     @mock.patch.object(co.Credential, 'get')
-    @mock.patch.object(context, 'get_service_context')
+    @mock.patch.object(context, 'get_service_credentials')
     @mock.patch.object(oslo_ctx, 'get_current')
-    def test_build_conn_params(self, mock_get_current, mock_get_service_ctx,
+    def test_build_conn_params(self, mock_get_current, mock_get_service_creds,
                                mock_cred_get):
         user = 'user1'
         project = 'project1'
@@ -386,7 +361,7 @@ class TestReceiver(base.SenlinTestCase):
 
         cred = mock.Mock()
         cred.cred = cred_info
-        mock_get_service_ctx.return_value = service_cred
+        mock_get_service_creds.return_value = service_cred
         mock_get_current.return_value = current_ctx
         mock_cred_get.return_value = cred
 
@@ -401,14 +376,14 @@ class TestReceiver(base.SenlinTestCase):
         }
         res = receiver._build_conn_params(user, project)
         self.assertEqual(expected_result, res)
-        mock_get_service_ctx.assert_called_once_with()
+        mock_get_service_creds.assert_called_once_with()
         mock_cred_get.assert_called_once_with(current_ctx, user, project)
 
     @mock.patch.object(co.Credential, 'get')
-    @mock.patch.object(context, 'get_service_context')
+    @mock.patch.object(context, 'get_service_credentials')
     @mock.patch.object(oslo_ctx, 'get_current')
     def test_build_conn_params_trust_not_found(
-            self, mock_get_current, mock_get_service_ctx, mock_cred_get):
+            self, mock_get_current, mock_get_service_creds, mock_cred_get):
 
         user = 'user1'
         project = 'project1'
@@ -419,12 +394,12 @@ class TestReceiver(base.SenlinTestCase):
             'password': '123'
         }
 
-        mock_get_service_ctx.return_value = service_cred
+        mock_get_service_creds.return_value = service_cred
         mock_cred_get.return_value = None
 
         receiver = self._create_receiver('receiver-1', UUID1)
         receiver = rb.Receiver.load(self.context, receiver_obj=receiver)
         ex = self.assertRaises(exception.TrustNotFound,
                                receiver._build_conn_params, user, project)
-        msg = "The trust for trustor (user1) could not be found."
-        self.assertEqual(msg, six.text_type(ex))
+        msg = "The trust for trustor 'user1' could not be found."
+        self.assertEqual(msg, str(ex))
