@@ -10,10 +10,10 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from unittest import mock
+
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
-import six
 
 from senlin.common import consts
 from senlin.common import exception
@@ -108,7 +108,7 @@ class TestNode(base.SenlinTestCase):
                                nodem.Node.load,
                                self.context, 'non-existent', None)
         self.assertEqual("The node 'non-existent' could not be found.",
-                         six.text_type(ex))
+                         str(ex))
 
         x_node_id = 'ee96c490-2dee-40c8-8919-4c64b89e326c'
         node = utils.create_node(self.context, x_node_id, PROFILE_ID,
@@ -145,7 +145,7 @@ class TestNode(base.SenlinTestCase):
                                nodem.Node.load,
                                new_ctx, x_node_id, None)
         self.assertEqual("The node '%s' could not be found." % x_node_id,
-                         six.text_type(ex))
+                         str(ex))
 
         res = nodem.Node.load(new_ctx, x_node_id, project_safe=False)
         self.assertIsNotNone(res)
@@ -251,11 +251,15 @@ class TestNode(base.SenlinTestCase):
                                     'Creation succeeded',
                                     physical_id=physical_id)
 
-    def test_node_create_not_init(self):
+    @mock.patch.object(nodem.Node, 'set_status')
+    def test_node_create_not_init(self, mock_status):
         node = nodem.Node('node1', PROFILE_ID, CLUSTER_ID, self.context)
         node.status = 'NOT_INIT'
-        res = node.do_create(self.context)
+        res, reason = node.do_create(self.context)
         self.assertFalse(res)
+        self.assertEqual('Node must be in INIT status', reason)
+        mock_status.assert_any_call(self.context, consts.NS_ERROR,
+                                    'Node must be in INIT status')
 
     @mock.patch.object(nodem.Node, 'set_status')
     @mock.patch.object(pb.Profile, 'create_object')
@@ -265,9 +269,10 @@ class TestNode(base.SenlinTestCase):
         mock_create.side_effect = exception.EResourceCreation(
             type='PROFILE', message='Boom', resource_id='test_id')
 
-        res = node.do_create(self.context)
+        res, reason = node.do_create(self.context)
 
         self.assertFalse(res)
+        self.assertEqual(str(reason), 'Failed in creating PROFILE: Boom.')
         mock_status.assert_any_call(self.context, consts.NS_CREATING,
                                     'Creation in progress')
         mock_status.assert_any_call(self.context, consts.NS_ERROR,
@@ -589,6 +594,24 @@ class TestNode(base.SenlinTestCase):
             % node.physical_id,
             physical_id=None)
 
+    @mock.patch.object(pb.Profile, 'healthcheck_object')
+    def test_node_healthcheck(self, mock_healthcheck):
+        node = nodem.Node('node1', PROFILE_ID, '')
+        node.status = consts.NS_ACTIVE
+        node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
+        mock_healthcheck.return_value = True
+        res = node.do_healthcheck(self.context)
+
+        self.assertTrue(res)
+        mock_healthcheck.assert_called_once_with(self.context, node)
+
+    def test_node_healthcheck_no_physical_id(self):
+        node = nodem.Node('node1', PROFILE_ID, '')
+
+        res = node.do_healthcheck(self.context)
+
+        self.assertFalse(res)
+
     @mock.patch.object(nodem.Node, 'set_status')
     @mock.patch.object(pb.Profile, 'recover_object')
     def test_node_recover_new_object(self, mock_recover, mock_status):
@@ -606,7 +629,7 @@ class TestNode(base.SenlinTestCase):
         mock_recover.return_value = new_id, True
         mock_status.side_effect = set_status
         action = mock.Mock()
-        action.inputs = {'operation': ['SWIM', 'DANCE']}
+        action.inputs = {'operation': 'SWIM'}
 
         res = node.do_recover(self.context, action)
 
@@ -667,7 +690,7 @@ class TestNode(base.SenlinTestCase):
         def set_status(*args, **kwargs):
             if args[1] == 'ACTIVE':
                 node.physical_id = new_id
-                node.data = {'recovery': 'RECREATE'}
+                node.data = {'recovery': 'recreate'}
 
         node = nodem.Node('node1', PROFILE_ID, '')
         node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
@@ -711,44 +734,7 @@ class TestNode(base.SenlinTestCase):
         mock_check = self.patchobject(pb.Profile, 'check_object')
         mock_check.return_value = False
         action = mock.Mock(
-            outputs={}, inputs={'operation': [{'name': 'RECREATE'}],
-                                'check': True})
-
-        res = node.do_recover(self.context, action)
-
-        self.assertTrue(res)
-        mock_check.assert_called_once_with(self.context, node)
-        mock_recover.assert_called_once_with(
-            self.context, node, **action.inputs)
-        self.assertEqual('node1', node.name)
-        self.assertEqual(new_id, node.physical_id)
-        self.assertEqual(PROFILE_ID, node.profile_id)
-        mock_status.assert_has_calls([
-            mock.call(self.context, 'RECOVERING',
-                      reason='Recovery in progress'),
-            mock.call(self.context, consts.NS_ACTIVE,
-                      reason='Recovery succeeded',
-                      physical_id=new_id,
-                      data={'recovery': 'RECREATE'})])
-
-    @mock.patch.object(nodem.Node, 'set_status')
-    @mock.patch.object(pb.Profile, 'recover_object')
-    def test_node_recover_mult_rebuild(self, mock_recover, mock_status):
-        def set_status(*args, **kwargs):
-            if args[1] == 'ACTIVE':
-                node.physical_id = new_id
-                node.data = {'recovery': 'RECREATE'}
-
-        node = nodem.Node('node1', PROFILE_ID, '', id='fake')
-        node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
-        new_id = '166db83b-b4a4-49ef-96a8-6c0fdd882d1a'
-        mock_recover.return_value = new_id, True
-        mock_status.side_effect = set_status
-        mock_check = self.patchobject(pb.Profile, 'check_object')
-        mock_check.return_value = False
-        action = mock.Mock(
-            outputs={}, inputs={'operation': [{'name': 'REBOOT'},
-                                              {'name': 'REBUILD'}],
+            outputs={}, inputs={'operation': 'RECREATE',
                                 'check': True})
 
         res = node.do_recover(self.context, action)
@@ -788,7 +774,7 @@ class TestNode(base.SenlinTestCase):
             id=node.physical_id,
             reason='Boom!'
         )
-        action = mock.Mock(inputs={'operation': ['boom'],
+        action = mock.Mock(inputs={'operation': 'boom',
                                    'check': True})
 
         res = node.do_recover(self.context, action)
@@ -814,7 +800,7 @@ class TestNode(base.SenlinTestCase):
         node = nodem.Node('node1', PROFILE_ID, None)
         node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
         mock_recover.return_value = node.physical_id, None
-        action = mock.Mock(inputs={'operation': [{'name': 'RECREATE'}]})
+        action = mock.Mock(inputs={'operation': 'RECREATE'})
 
         res = node.do_recover(self.context, action)
 
@@ -825,20 +811,178 @@ class TestNode(base.SenlinTestCase):
             mock.call(self.context, consts.NS_ERROR,
                       reason='Recovery failed')])
 
-    def test_node_recover_no_physical_id(self):
+    @mock.patch.object(nodem.Node, 'set_status')
+    @mock.patch.object(pb.Profile, 'recover_object')
+    def test_node_recover_failed_recover_with_old_physical_id(self,
+                                                              mock_recover,
+                                                              mock_status):
         node = nodem.Node('node1', PROFILE_ID, None)
-        action = mock.Mock()
+        node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
+
+        action = mock.Mock(
+            inputs={'operation': consts.RECOVER_RECREATE, 'check': True})
+        mock_recover.side_effect = exception.EResourceOperation(
+            op=consts.RECOVER_RECREATE,
+            type='server',
+            id=node.physical_id,
+            resource_id=node.physical_id,
+            reason='Recovery failed',
+        )
+        res = node.do_recover(self.context, action)
+
+        self.assertFalse(res)
+        mock_recover.assert_called_once_with(
+            self.context, node, **action.inputs)
+        reason = ("Failed in RECREATE server 'd94d6333-82e6-4f87-b7ab-b786776d"
+                  "f9d1': Internal error happened.")
+        mock_status.assert_has_calls([
+            mock.call(self.context, 'RECOVERING',
+                      reason='Recovery in progress'),
+            mock.call(self.context, consts.NS_ERROR,
+                      reason=str(reason),
+                      physical_id=node.physical_id)])
+
+    @mock.patch.object(nodem.Node, 'set_status')
+    @mock.patch.object(pb.Profile, 'recover_object')
+    def test_node_recover_failed_recover_with_new_physical_id(self,
+                                                              mock_recover,
+                                                              mock_status):
+        def set_status(*args, **kwargs):
+            if args[1] == consts.NS_ERROR:
+                node.physical_id = new_id
+
+        node = nodem.Node('node1', PROFILE_ID, None)
+        node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
+        new_id = '166db83b-b4a4-49ef-96a8-6c0fdd882d1a'
+
+        mock_status.side_effect = set_status
+        action = mock.Mock(inputs={'operation': consts.RECOVER_RECREATE,
+                                   'check': True})
+        mock_recover.side_effect = exception.EResourceOperation(
+            op=consts.RECOVER_RECREATE,
+            type='server',
+            id=node.physical_id,
+            resource_id=new_id,
+            reason='Recovery failed',
+        )
+        res = node.do_recover(self.context, action)
+
+        self.assertFalse(res)
+        mock_recover.assert_called_once_with(
+            self.context, node, **action.inputs)
+        reason = ("Failed in RECREATE server 'd94d6333-82e6-4f87-b7ab-b786776d"
+                  "f9d1': Internal error happened.")
+        mock_status.assert_has_calls([
+            mock.call(self.context, 'RECOVERING',
+                      reason='Recovery in progress'),
+            mock.call(self.context, consts.NS_ERROR,
+                      reason=str(reason),
+                      physical_id=new_id)])
+
+    def test_node_recover_no_physical_id_reboot_op(self):
+        node = nodem.Node('node1', PROFILE_ID, None)
+        action = mock.Mock(inputs={'operation': 'REBOOT'})
+
+        res = node.do_recover(self.context, action)
+
+        self.assertFalse(res)
+
+    def test_node_recover_no_physical_id_rebuild_op(self):
+        node = nodem.Node('node1', PROFILE_ID, None)
+        action = mock.Mock(inputs={'operation': 'REBUILD'})
 
         res = node.do_recover(self.context, action)
 
         self.assertFalse(res)
 
+    @mock.patch.object(nodem.Node, 'set_status')
+    @mock.patch.object(pb.Profile, 'recover_object')
+    def test_node_recover_no_physical_id_no_op(self, mock_recover,
+                                               mock_status):
+        def set_status(*args, **kwargs):
+            if args[1] == 'ACTIVE':
+                node.physical_id = new_id
+                node.data = {'recovery': 'RECREATE'}
+
+        node = nodem.Node('node1', PROFILE_ID, '', id='fake')
+        new_id = '166db83b-b4a4-49ef-96a8-6c0fdd882d1a'
+        mock_recover.return_value = new_id, True
+        mock_status.side_effect = set_status
+        mock_check = self.patchobject(pb.Profile, 'check_object')
+        mock_check.return_value = False
+        action = mock.Mock(
+            outputs={}, inputs={})
+
+        res = node.do_recover(self.context, action)
+
+        self.assertTrue(res)
+        mock_check.assert_not_called()
+        mock_recover.assert_called_once_with(
+            self.context, node, **action.inputs)
+        self.assertEqual('node1', node.name)
+        self.assertEqual(new_id, node.physical_id)
+        self.assertEqual(PROFILE_ID, node.profile_id)
+        mock_status.assert_has_calls([
+            mock.call(self.context, 'RECOVERING',
+                      reason='Recovery in progress'),
+            mock.call(self.context, consts.NS_ACTIVE,
+                      reason='Recovery succeeded',
+                      physical_id=new_id,
+                      data={'recovery': 'RECREATE'})])
+
+    @mock.patch.object(nodem.Node, 'set_status')
+    @mock.patch.object(pb.Profile, 'recover_object')
+    def test_node_recover_no_physical_id_recreate_op(self, mock_recover,
+                                                     mock_status):
+        def set_status(*args, **kwargs):
+            if args[1] == 'ACTIVE':
+                node.physical_id = new_id
+                node.data = {'recovery': 'RECREATE'}
+
+        node = nodem.Node('node1', PROFILE_ID, '', id='fake')
+        new_id = '166db83b-b4a4-49ef-96a8-6c0fdd882d1a'
+        mock_recover.return_value = new_id, True
+        mock_status.side_effect = set_status
+        mock_check = self.patchobject(pb.Profile, 'check_object')
+        mock_check.return_value = False
+        action = mock.Mock(
+            outputs={}, inputs={'operation': 'RECREATE',
+                                'check': True})
+
+        res = node.do_recover(self.context, action)
+
+        self.assertTrue(res)
+        mock_check.assert_called_once_with(self.context, node)
+        mock_recover.assert_called_once_with(
+            self.context, node, **action.inputs)
+        self.assertEqual('node1', node.name)
+        self.assertEqual(new_id, node.physical_id)
+        self.assertEqual(PROFILE_ID, node.profile_id)
+        mock_status.assert_has_calls([
+            mock.call(self.context, 'RECOVERING',
+                      reason='Recovery in progress'),
+            mock.call(self.context, consts.NS_ACTIVE,
+                      reason='Recovery succeeded',
+                      physical_id=new_id,
+                      data={'recovery': 'RECREATE'})])
+
     @mock.patch.object(nodem.Node, 'set_status')
     def test_node_recover_operation_not_support(self, mock_set_status):
         node = nodem.Node('node1', PROFILE_ID, None)
         node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
         action = mock.Mock(
-            outputs={}, inputs={'operation': [{'name': 'foo'}]})
+            outputs={}, inputs={'operation': 'foo'})
+
+        res = node.do_recover(self.context, action)
+        self.assertEqual({}, action.outputs)
+        self.assertFalse(res)
+
+    @mock.patch.object(nodem.Node, 'set_status')
+    def test_node_recover_operation_not_string(self, mock_set_status):
+        node = nodem.Node('node1', PROFILE_ID, None)
+        node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
+        action = mock.Mock(
+            outputs={}, inputs={'operation': 'foo'})
 
         res = node.do_recover(self.context, action)
         self.assertEqual({}, action.outputs)
@@ -894,6 +1038,28 @@ class TestNode(base.SenlinTestCase):
         ])
         x_profile.handle_dance.assert_called_once_with(node, style='tango')
 
+    def _verify_execution_create_args(self, expected_name,
+                                      expected_inputs_dict, wfc):
+        wfc.execution_create.assert_called_once_with(mock.ANY, mock.ANY)
+        actual_call_args, call_kwargs = wfc.execution_create.call_args
+
+        # execution_create parameters are name and inputs
+        actual_call_name, actual_call_inputs = actual_call_args
+
+        # actual_call_inputs is string representation of a dictionary.
+        # convert actual_call_inputs to json, then dump it back as string
+        # sorted by key
+        final_actual_call_inputs = jsonutils.dumps(
+            jsonutils.loads(actual_call_inputs), sort_keys=True)
+
+        # dump expected_inputs_dict as string sorted by key
+        final_expected_inputs = jsonutils.dumps(
+            expected_inputs_dict, sort_keys=True)
+
+        # compare the sorted input strings along with the names
+        self.assertEqual(actual_call_name, expected_name)
+        self.assertEqual(final_actual_call_inputs, final_expected_inputs)
+
     def test_run_workflow(self):
         node = nodem.Node('node1', PROFILE_ID, 'FAKE_CLUSTER')
         node.physical_id = 'FAKE_NODE'
@@ -931,10 +1097,7 @@ class TestNode(base.SenlinTestCase):
             'FAKE_KEY1': 'FAKE_VALUE1',
             'FAKE_KEY2': 'FAKE_VALUE2',
         }
-        final_inputs = jsonutils.dumps(final_dict)
-        wfc.execution_create.assert_called_once_with(mock.ANY, mock.ANY)
-        call_args, call_kwargs = wfc.execution_create.call_args
-        self.assertEqual(call_args, ('foo', final_inputs))
+        self._verify_execution_create_args('foo', final_dict, wfc)
 
     def test_run_workflow_no_physical_id(self):
         node = nodem.Node('node1', PROFILE_ID, 'FAKE_CLUSTER')
@@ -976,10 +1139,7 @@ class TestNode(base.SenlinTestCase):
             'FAKE_KEY1': 'FAKE_VALUE1',
             'FAKE_KEY2': 'FAKE_VALUE2',
         }
-        final_inputs = jsonutils.dumps(final_dict)
-        wfc.execution_create.assert_called_once_with(mock.ANY, mock.ANY)
-        call_args, call_kwargs = wfc.execution_create.call_args
-        self.assertEqual(call_args, ('foo', final_inputs))
+        self._verify_execution_create_args('foo', final_dict, wfc)
 
     def test_run_workflow_failed_creation(self):
         node = nodem.Node('node1', PROFILE_ID, 'FAKE_CLUSTER')
@@ -1008,7 +1168,7 @@ class TestNode(base.SenlinTestCase):
                                **options)
 
         self.assertEqual("Failed in executing workflow 'foo': boom.",
-                         six.text_type(ex))
+                         str(ex))
         x_profile.workflow.assert_called_once_with(node)
         wfc.workflow_find.assert_called_once_with('foo')
         wfc.workflow_create.assert_called_once_with(
@@ -1042,7 +1202,7 @@ class TestNode(base.SenlinTestCase):
                                **options)
 
         self.assertEqual("Failed in executing workflow 'foo': boom.",
-                         six.text_type(ex))
+                         str(ex))
         x_profile.workflow.assert_called_once_with(node)
         wfc.workflow_find.assert_called_once_with('foo')
         wfc.workflow_create.assert_called_once_with(
@@ -1053,7 +1213,4 @@ class TestNode(base.SenlinTestCase):
             'FAKE_KEY1': 'FAKE_VALUE1',
             'FAKE_KEY2': 'FAKE_VALUE2',
         }
-        final_inputs = jsonutils.dumps(final_dict)
-        wfc.execution_create.assert_called_once_with(mock.ANY, mock.ANY)
-        call_args, call_kwargs = wfc.execution_create.call_args
-        self.assertEqual(call_args, ('foo', final_inputs))
+        self._verify_execution_create_args('foo', final_dict, wfc)
