@@ -10,10 +10,10 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from unittest import mock
+
 from oslo_context import context as oslo_ctx
 from oslo_utils import timeutils
-import six
 
 from senlin.common import consts
 from senlin.common import context as senlin_ctx
@@ -65,13 +65,15 @@ class TestPolicyBase(base.SenlinTestCase):
         self.ctx = utils.dummy_context()
         environment.global_env().register_policy('senlin.policy.dummy-1.0',
                                                  DummyPolicy)
+        environment.global_env().register_policy('senlin.policy.dummy-1.1',
+                                                 DummyPolicy)
         self.spec = parser.simple_parse(sample_policy)
 
     def _create_policy(self, policy_name, policy_id=None):
         policy = pb.Policy(policy_name, self.spec,
-                           user=self.ctx.user,
-                           project=self.ctx.project,
-                           domain=self.ctx.domain)
+                           user=self.ctx.user_id,
+                           project=self.ctx.project_id,
+                           domain=self.ctx.domain_id)
         if policy_id:
             policy.id = policy_id
 
@@ -83,9 +85,9 @@ class TestPolicyBase(base.SenlinTestCase):
             'type': 'senlin.policy.dummy-1.0',
             'spec': self.spec,
             'created_at': timeutils.utcnow(True),
-            'user': self.ctx.user,
-            'project': self.ctx.project,
-            'domain': self.ctx.domain,
+            'user': self.ctx.user_id,
+            'project': self.ctx.project_id,
+            'domain': self.ctx.domain_id,
         }
 
         values.update(kwargs)
@@ -98,9 +100,9 @@ class TestPolicyBase(base.SenlinTestCase):
         self.assertEqual('test-policy', policy.name)
         self.assertEqual(self.spec, policy.spec)
         self.assertEqual('senlin.policy.dummy-1.0', policy.type)
-        self.assertEqual(self.ctx.user, policy.user)
-        self.assertEqual(self.ctx.project, policy.project)
-        self.assertEqual(self.ctx.domain, policy.domain)
+        self.assertEqual(self.ctx.user_id, policy.user)
+        self.assertEqual(self.ctx.project_id, policy.project)
+        self.assertEqual(self.ctx.domain_id, policy.domain)
         self.assertEqual({}, policy.data)
         self.assertIsNone(policy.created_at)
         self.assertIsNone(policy.updated_at)
@@ -113,6 +115,52 @@ class TestPolicyBase(base.SenlinTestCase):
                          spec_data['properties'])
         self.assertEqual({'key1': 'value1', 'key2': 2}, policy.properties)
 
+    def test_init_version_as_float(self):
+        self.spec['version'] = 1.1
+        policy = self._create_policy('test-policy')
+
+        self.assertIsNone(policy.id)
+        self.assertEqual('test-policy', policy.name)
+        self.assertEqual(self.spec, policy.spec)
+        self.assertEqual('senlin.policy.dummy-1.1', policy.type)
+        self.assertEqual(self.ctx.user_id, policy.user)
+        self.assertEqual(self.ctx.project_id, policy.project)
+        self.assertEqual(self.ctx.domain_id, policy.domain)
+        self.assertEqual({}, policy.data)
+        self.assertIsNone(policy.created_at)
+        self.assertIsNone(policy.updated_at)
+        self.assertTrue(policy.singleton)
+
+        spec_data = policy.spec_data
+        self.assertEqual('senlin.policy.dummy', spec_data['type'])
+        self.assertEqual('1.1', spec_data['version'])
+        self.assertEqual({'key1': 'value1', 'key2': 2},
+                         spec_data['properties'])
+        self.assertEqual({'key1': 'value1', 'key2': 2}, policy.properties)
+
+    def test_init_version_as_string(self):
+        self.spec['version'] = '1.1'
+        policy = self._create_policy('test-policy')
+
+        self.assertIsNone(policy.id)
+        self.assertEqual('test-policy', policy.name)
+        self.assertEqual(self.spec, policy.spec)
+        self.assertEqual('senlin.policy.dummy-1.1', policy.type)
+        self.assertEqual(self.ctx.user_id, policy.user)
+        self.assertEqual(self.ctx.project_id, policy.project)
+        self.assertEqual(self.ctx.domain_id, policy.domain)
+        self.assertEqual({}, policy.data)
+        self.assertIsNone(policy.created_at)
+        self.assertIsNone(policy.updated_at)
+        self.assertTrue(policy.singleton)
+
+        spec_data = policy.spec_data
+        self.assertEqual('senlin.policy.dummy', spec_data['type'])
+        self.assertEqual('1.1', spec_data['version'])
+        self.assertEqual({'key1': 'value1', 'key2': 2},
+                         spec_data['properties'])
+        self.assertEqual({'key1': 'value1', 'key2': 2}, policy.properties)
+
     def test_policy_new_type_not_found(self):
         bad_spec = {
             'type': 'bad-type',
@@ -166,55 +214,14 @@ class TestPolicyBase(base.SenlinTestCase):
         ex = self.assertRaises(exception.ResourceNotFound,
                                pb.Policy.load,
                                self.ctx, 'fake-policy', None)
-        self.assertEqual('The policy (fake-policy) could not be found.',
-                         six.text_type(ex))
+        self.assertEqual("The policy 'fake-policy' could not be found.",
+                         str(ex))
 
         ex = self.assertRaises(exception.ResourceNotFound,
                                pb.Policy.load,
                                self.ctx, None, None)
-        self.assertEqual('The policy (None) could not be found.',
-                         six.text_type(ex))
-
-    def test_load_all(self):
-        result = pb.Policy.load_all(self.ctx)
-        self.assertEqual([], list(result))
-
-        policy1 = utils.create_policy(self.ctx, UUID1, 'policy-1')
-        policy2 = utils.create_policy(self.ctx, UUID2, 'policy-2')
-
-        result = pb.Policy.load_all(self.ctx)
-        policies = list(result)
-        self.assertEqual(2, len(policies))
-        self.assertEqual(policy1.id, policies[0].id)
-        self.assertEqual(policy2.id, policies[1].id)
-
-    def test_load_all_diff_project(self):
-        utils.create_policy(self.ctx, UUID1, 'policy-1')
-        utils.create_policy(self.ctx, UUID2, 'policy-2')
-
-        new_ctx = utils.dummy_context(project='a-different-project')
-        res = pb.Policy.load_all(new_ctx)
-        self.assertEqual(0, len(list(res)))
-        res = pb.Policy.load_all(new_ctx, project_safe=False)
-        self.assertEqual(2, len(list(res)))
-
-    @mock.patch.object(po.Policy, 'get_all')
-    def test_load_all_with_params(self, mock_get):
-        mock_get.return_value = []
-
-        res = list(pb.Policy.load_all(self.ctx))
-        self.assertEqual([], res)
-        mock_get.assert_called_once_with(self.ctx, limit=None, marker=None,
-                                         sort=None, filters=None,
-                                         project_safe=True)
-        mock_get.reset_mock()
-
-        res = list(pb.Policy.load_all(
-            self.ctx, limit=1, marker='MARKER', sort='K1:asc'))
-        self.assertEqual([], res)
-        mock_get.assert_called_once_with(self.ctx, limit=1, marker='MARKER',
-                                         sort='K1:asc', filters=None,
-                                         project_safe=True)
+        self.assertEqual("The policy 'None' could not be found.",
+                         str(ex))
 
     def test_delete(self):
         policy = utils.create_policy(self.ctx, UUID1)
@@ -522,9 +529,9 @@ class TestPolicyBase(base.SenlinTestCase):
         self.assertEqual((True, None), res)
 
     @mock.patch.object(co.Credential, 'get')
-    @mock.patch.object(senlin_ctx, 'get_service_context')
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
     @mock.patch.object(oslo_ctx, 'get_current')
-    def test_build_conn_params(self, mock_get_current, mock_get_service_ctx,
+    def test_build_conn_params(self, mock_get_current, mock_get_service_creds,
                                mock_cred_get):
         service_cred = {
             'auth_url': 'AUTH_URL',
@@ -545,7 +552,7 @@ class TestPolicyBase(base.SenlinTestCase):
         }
 
         cred = mock.Mock(cred=cred_info)
-        mock_get_service_ctx.return_value = service_cred
+        mock_get_service_creds.return_value = service_cred
         mock_get_current.return_value = current_ctx
         mock_cred_get.return_value = cred
         policy = self._create_policy('test-policy')
@@ -560,14 +567,14 @@ class TestPolicyBase(base.SenlinTestCase):
             'trust_id': 'TRUST_ID'
         }
         self.assertEqual(expected_result, res)
-        mock_get_service_ctx.assert_called_once_with()
+        mock_get_service_creds.assert_called_once_with()
         mock_cred_get.assert_called_once_with(current_ctx, 'user1', 'project1')
 
     @mock.patch.object(co.Credential, 'get')
-    @mock.patch.object(senlin_ctx, 'get_service_context')
+    @mock.patch.object(senlin_ctx, 'get_service_credentials')
     @mock.patch.object(oslo_ctx, 'get_current')
     def test_build_conn_params_trust_not_found(
-            self, mock_get_current, mock_get_service_ctx, mock_cred_get):
+            self, mock_get_current, mock_get_service_creds, mock_cred_get):
 
         service_cred = {
             'auth_url': 'AUTH_URL',
@@ -576,7 +583,7 @@ class TestPolicyBase(base.SenlinTestCase):
             'password': '123'
         }
 
-        mock_get_service_ctx.return_value = service_cred
+        mock_get_service_creds.return_value = service_cred
         mock_cred_get.return_value = None
         policy = self._create_policy('test-policy')
 
@@ -584,5 +591,5 @@ class TestPolicyBase(base.SenlinTestCase):
                                policy._build_conn_params,
                                'user1', 'project1')
 
-        msg = "The trust for trustor (user1) could not be found."
-        self.assertEqual(msg, six.text_type(ex))
+        msg = "The trust for trustor 'user1' could not be found."
+        self.assertEqual(msg, str(ex))
