@@ -10,8 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
-import six
+from unittest import mock
 
 from senlin.common import exception as exc
 from senlin.profiles.os.nova import server
@@ -117,7 +116,7 @@ class TestAvailabilityZoneValidation(base.SenlinTestCase):
             ex = self.assertRaises(self.exception,
                                    self.profile._validate_az,
                                    node, 'FAKE_AZ', self.reason)
-            self.assertEqual(self.message, six.text_type(ex))
+            self.assertEqual(self.message, str(ex))
 
         self.cc.validate_azs.assert_called_once_with(['FAKE_AZ'])
 
@@ -233,7 +232,7 @@ class TestFlavorValidation(base.SenlinTestCase):
             ex = self.assertRaises(self.exception,
                                    self.profile._validate_flavor,
                                    node, flavor, self.reason)
-            self.assertEqual(self.message, six.text_type(ex))
+            self.assertEqual(self.message, str(ex))
 
         self.cc.flavor_find.assert_called_once_with(flavor, False)
 
@@ -310,11 +309,13 @@ class TestImageValidation(base.SenlinTestCase):
         super(TestImageValidation, self).setUp()
 
         self.cc = mock.Mock()
+        self.gc = mock.Mock()
         self.profile = server.ServerProfile('t', spec)
         self.profile._computeclient = self.cc
+        self.profile._glanceclient = self.gc
 
     def test_validation(self):
-        self.cc.image_find.side_effect = self.validate_result
+        self.gc.image_find.side_effect = self.validate_result
         node = mock.Mock(id='NODE_ID', physical_id='NOVA_ID')
         image = 'IMAGE'
 
@@ -326,9 +327,9 @@ class TestImageValidation(base.SenlinTestCase):
             ex = self.assertRaises(self.exception,
                                    self.profile._validate_image,
                                    node, image, self.reason)
-            self.assertEqual(self.message, six.text_type(ex))
+            self.assertEqual(self.message, str(ex))
 
-        self.cc.image_find.assert_called_once_with(image, False)
+        self.gc.image_find.assert_called_once_with(image, False)
 
 
 class TestVolumeValidation(base.SenlinTestCase):
@@ -426,7 +427,7 @@ class TestVolumeValidation(base.SenlinTestCase):
             ex = self.assertRaises(self.exception,
                                    self.profile._validate_volume,
                                    node, volume, self.reason)
-            self.assertEqual(self.message, six.text_type(ex))
+            self.assertEqual(self.message, str(ex))
 
         self.vc.volume_get.assert_called_once_with(volume)
 
@@ -519,7 +520,7 @@ class TestKeypairValidation(base.SenlinTestCase):
             ex = self.assertRaises(self.exception,
                                    self.profile._validate_keypair,
                                    node, key, self.reason)
-            self.assertEqual(self.message, six.text_type(ex))
+            self.assertEqual(self.message, str(ex))
 
         self.cc.keypair_find.assert_called_once_with(key, False)
 
@@ -894,14 +895,44 @@ class TestNetworkValidation(base.SenlinTestCase):
             ex = self.assertRaises(self.exception,
                                    self.profile._validate_network,
                                    obj, self.inputs, self.reason)
-            self.assertEqual(self.message, six.text_type(ex))
+            self.assertEqual(self.message, str(ex))
 
         if self.net_result:
             self.nc.network_get.assert_called_with('NET')
         if self.port_result:
             self.nc.port_find.assert_called_once_with('PORT')
         if self.sg_result:
-            self.nc.security_group_find.assert_called_once_with('default')
+            self.nc.security_group_find.assert_called_once_with(
+                'default', project_id=None)
+        if self.floating_result:
+            self.nc.floatingip_find.assert_called_once_with('FLOATINGIP')
+
+    def test_validation_with_project_scope(self):
+        self.nc.network_get.side_effect = self.net_result
+        self.nc.port_find.side_effect = self.port_result
+        self.nc.security_group_find.side_effect = self.sg_result
+        self.nc.floatingip_find.side_effect = self.floating_result
+        obj = mock.Mock(physical_id='NOVA_ID')
+
+        self.profile.project_scope = True
+        self.profile.project = 'FAKE_PROJECT_ID'
+
+        if self.success:
+            res = self.profile._validate_network(obj, self.inputs, self.reason)
+            self.assertEqual(self.result, res)
+        else:
+            ex = self.assertRaises(self.exception,
+                                   self.profile._validate_network,
+                                   obj, self.inputs, self.reason)
+            self.assertEqual(self.message, str(ex))
+
+        if self.net_result:
+            self.nc.network_get.assert_called_with('NET')
+        if self.port_result:
+            self.nc.port_find.assert_called_once_with('PORT')
+        if self.sg_result:
+            self.nc.security_group_find.assert_called_once_with(
+                'default', project_id='FAKE_PROJECT_ID')
         if self.floating_result:
             self.nc.floatingip_find.assert_called_once_with('FLOATINGIP')
 
