@@ -11,11 +11,13 @@
 # under the License.
 
 import datetime
-import mock
+import io
+from unittest import mock
+import urllib
+
 from oslo_log import log as logging
 from oslo_utils import timeutils
 import requests
-import six
 
 from oslo_config import cfg
 
@@ -25,89 +27,20 @@ from senlin.objects import service as service_obj
 from senlin.tests.unit.common import base
 
 
-class TestParameterParsing(base.SenlinTestCase):
-    def test_parse_bool(self):
-        name = 'param'
-        for value in ('True', 'true', 'TRUE', True):
-            self.assertTrue(utils.parse_bool_param(name, value))
-        for value in ('False', 'false', 'FALSE', False):
-            self.assertFalse(utils.parse_bool_param(name, value))
-        for value in ('foo', 't', 'f', 'yes', 'no', 'y', 'n', '1', '0', None):
-            self.assertRaises(exception.InvalidParameter,
-                              utils.parse_bool_param, name, value)
+class TestGetPositiveInt(base.SenlinTestCase):
 
-    def test_parse_int(self):
-        name = 'param'
-        cases = {0: 0, 2: 2, '0': 0, '2': 2}
+    def test_get_positive_int(self):
+        cases = {1: 1, 2: 2, '1': 1, '2': 2}
         for value, expected in cases.items():
-            actual = utils.parse_int_param(name, value)
+            res, actual = utils.get_positive_int(value)
+            self.assertTrue(res)
             self.assertEqual(expected, actual)
 
-        # A None should be returned directly
-        actual = utils.parse_int_param(name, None)
-        self.assertIsNone(actual)
-
-        for value in (2, '2'):
-            self.assertTrue(utils.parse_int_param(name, value,
-                                                  allow_zero=False))
-        for value in (0, '0'):
-            self.assertRaises(exception.InvalidParameter,
-                              utils.parse_int_param, name, value,
-                              allow_zero=False)
-        for value in (-1, '-2'):
-            self.assertTrue(utils.parse_int_param(name, value,
-                                                  allow_negative=True))
-        for value in (-1, '-2'):
-            self.assertRaises(exception.InvalidParameter,
-                              utils.parse_int_param, name, value)
-
-        for value in (1, 6):
-            self.assertRaises(exception.InvalidParameter,
-                              utils.parse_int_param, name, value,
-                              lower_limit=2, upper_limit=5)
-
-    def test_validate_sort_param(self):
-        whitelist = ['foo', 'bar', 'zoo']
-        # None case
-        actual = utils.validate_sort_param(None, whitelist)
-        self.assertIsNone(actual)
-
-        # single good key
-        actual = utils.validate_sort_param('foo', whitelist)
-        self.assertIsNone(actual)
-
-        # multiple keys
-        actual = utils.validate_sort_param('foo,bar', whitelist)
-        self.assertIsNone(actual)
-
-        # with dirs
-        value = 'foo:asc,bar,zoo:desc'
-        actual = utils.validate_sort_param(value, whitelist)
-        self.assertIsNone(actual)
-
-    def test_validate_sort_param_key_missing(self):
-        whitelist = ['foo', 'bar', 'zoo']
-        ex = self.assertRaises(exception.InvalidParameter,
-                               utils.validate_sort_param,
-                               ':asc', whitelist)
-        self.assertEqual("Invalid value '' specified for 'sort key'",
-                         six.text_type(ex))
-
-    def test_validate_sort_param_invalid_key(self):
-        whitelist = ['foo', 'bar', 'zoo']
-        ex = self.assertRaises(exception.InvalidParameter,
-                               utils.validate_sort_param,
-                               'cool', whitelist)
-        self.assertEqual("Invalid value 'cool' specified for 'sort key'",
-                         six.text_type(ex))
-
-    def test_validate_sort_param_invalid_dir(self):
-        whitelist = ['foo', 'bar', 'zoo']
-        ex = self.assertRaises(exception.InvalidParameter,
-                               utils.validate_sort_param,
-                               'bar:inc', whitelist)
-        self.assertEqual("Invalid value 'inc' specified for 'sort dir'",
-                         six.text_type(ex))
+        bad_values = ['foo', {}, [], -1, 1.5, 0.2, None]
+        for value in bad_values:
+            res, actual = utils.get_positive_int(value)
+            self.assertFalse(res)
+            self.assertEqual(0, actual)
 
 
 class Response(object):
@@ -128,19 +61,19 @@ class UrlFetchTest(base.SenlinTestCase):
         self.assertRaises(utils.URLFetchError,
                           utils.url_fetch, 'file:///etc/profile')
 
-    def test_file_scheme_supported(self):
+    @mock.patch('urllib.request.urlopen')
+    def test_file_scheme_supported(self, mock_urlopen):
         data = '{ "foo": "bar" }'
         url = 'file:///etc/profile'
+        mock_urlopen.return_value = io.StringIO(data)
 
-        self.patchobject(six.moves.urllib.request, 'urlopen',
-                         return_value=six.moves.cStringIO(data))
         actual = utils.url_fetch(url, allowed_schemes=['file'])
         self.assertEqual(data, actual)
 
-    def test_file_scheme_failure(self):
+    @mock.patch('urllib.request.urlopen')
+    def test_file_scheme_failure(self, mock_urlopen):
         url = 'file:///etc/profile'
-        self.patchobject(six.moves.urllib.request, 'urlopen',
-                         side_effect=six.moves.urllib.error.URLError('oops'))
+        mock_urlopen.side_effect = urllib.error.URLError('oops')
 
         self.assertRaises(utils.URLFetchError,
                           utils.url_fetch, url, allowed_schemes=['file'])
@@ -178,18 +111,40 @@ class UrlFetchTest(base.SenlinTestCase):
     def test_max_fetch_size_okay(self):
         url = 'http://example.com/somedata'
         data = '{ "foo": "bar" }'
-        cfg.CONF.set_override('max_response_size', 500, enforce_type=True)
+        cfg.CONF.set_override('max_response_size', 500)
         self.patchobject(requests, 'get', return_value=Response(data))
         utils.url_fetch(url)
 
     def test_max_fetch_size_error(self):
         url = 'http://example.com/somedata'
         data = '{ "foo": "bar" }'
-        cfg.CONF.set_override('max_response_size', 5, enforce_type=True)
+        cfg.CONF.set_override('max_response_size', 5)
         self.patchobject(requests, 'get', return_value=Response(data))
         exception = self.assertRaises(utils.URLFetchError,
                                       utils.url_fetch, url)
-        self.assertIn("Data exceeds", six.text_type(exception))
+        self.assertIn("Data exceeds", str(exception))
+
+    @mock.patch.object(requests, 'get')
+    def test_string_response(self, mock_get):
+        url = 'http://example.com/somedata'
+        data = '{ "foo": "bar" }'
+
+        mock_resp = mock.Mock()
+        mock_resp.iter_content.return_value = [data]
+        mock_get.return_value = mock_resp
+
+        self.assertEqual(data, utils.url_fetch(url))
+
+    @mock.patch.object(requests, 'get')
+    def test_byte_response(self, mock_get):
+        url = 'http://example.com/somedata'
+        data = b'{ "foo": "bar" }'
+
+        mock_resp = mock.Mock()
+        mock_resp.iter_content.return_value = [data]
+        mock_get.return_value = mock_resp
+
+        self.assertEqual('{ "foo": "bar" }', utils.url_fetch(url))
 
 
 class TestRandomName(base.SenlinTestCase):
@@ -224,6 +179,39 @@ class TestRandomName(base.SenlinTestCase):
         self.assertEqual('', result)
 
 
+class TestFormatNodeName(base.SenlinTestCase):
+
+    def test_empty(self):
+        res = utils.format_node_name(None, None, 0)
+        self.assertIsNotNone(res)
+        self.assertEqual(13, len(res))
+
+        res = utils.format_node_name("", None, 0)
+        self.assertIsNotNone(res)
+        self.assertEqual(13, len(res))
+
+    def test_has_random(self):
+        res = utils.format_node_name("prefix-$R", None, 0)
+        self.assertEqual(15, len(res))
+
+        res = utils.format_node_name("prefix-$5R", None, 0)
+        self.assertEqual(12, len(res))
+
+    def test_has_index(self):
+        res = utils.format_node_name("prefix-$I", None, 12)
+        self.assertEqual(9, len(res))
+
+        res = utils.format_node_name("prefix-$5I", None, 12)
+        self.assertEqual(12, len(res))
+
+    def test_has_both(self):
+        res = utils.format_node_name("prefix-$3R-$I", None, 12)
+        self.assertEqual(13, len(res))
+
+        res = utils.format_node_name("$3R-prefix-$5I", None, 12)
+        self.assertEqual(16, len(res))
+
+
 class TestParseLevelValues(base.SenlinTestCase):
 
     def test_none(self):
@@ -239,7 +227,7 @@ class TestParseLevelValues(base.SenlinTestCase):
         self.assertEqual([logging.ERROR], res)
 
     def test_multi_values(self):
-        res = utils.parse_level_values(['WARNING', 'ERROR'])
+        res = utils.parse_level_values(['WARN', 'ERROR'])
         self.assertEqual([logging.WARNING, logging.ERROR], res)
 
     def test_with_invalid_values(self):
@@ -265,9 +253,8 @@ class TestGetPathParser(base.SenlinTestCase):
         err = self.assertRaises(exception.BadRequest,
                                 utils.get_path_parser,
                                 '^foo.bar')
-        self.assertEqual("The request is malformed: Invalid attribute path - "
-                         "Unexpected character: ^.",
-                         six.text_type(err))
+        self.assertEqual("Invalid attribute path - Unexpected "
+                         "character: ^.", str(err))
 
 
 class EngineDeathTest(base.SenlinTestCase):
